// ****************************************************************************
//
// Copyright ?2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************

function adController ()
{
    // default max ad refresh interval is 5 minutes
    this.maxRefreshInterval = 5*60*1000;
    // default min ad refresh interval is 30 seconds
    this.minRefreshInterval = 30*1000;
    this.triggerThreshold = 5;
}

adController.prototype.init = function()
{
    this.adRefreshTimer = null;
    this.actionCounter = 0;
    var d = new Date();
    this.lastRefreshTime = d.getTime();
    this.setRefreshTimer();
}

adController.prototype.release = function()
{
    if (this.adRefreshTimer)
    {
        this.adRefreshTimer.cancel();
    }
    this.adRefreshTimer = null;
    this.actionCounter = 0;
    this.lastRefreshTime = 0;
}

adController.prototype.trigger = function()
{
    this.actionCounter++;
    if (this.actionCounter >= this.triggerThreshold)
    {
        var d = new Date();
        var lastRefreshDelta = d.getTime() - this.lastRefreshTime;
        if (lastRefreshDelta >= this.minRefreshInterval)
        {
            this.refreshAds();
        }
    }
}

adController.prototype.refreshAds = function()
{
    if (scene.rootBox.active)
    {
        var d = new Date();
        this.actionCounter = 0;
        this.lastRefreshTime = d.getTime();
        var convBar = scene.getBoxById("activeConversationBar");
        if (convBar && !convBar.collapsed)
        {
            var convAd = convBar.getPartById("convAd");
            if (convAd)
            {
                convAd.getPartById("icon").TSMretrieveAd();
            }
        }
    }
    this.setRefreshTimer();
}

adController.prototype.setRefreshTimer = function(inTime)
{
    if (this.adRefreshTimer)
    {
        this.adRefreshTimer.cancel();
        this.adRefreshTimer = null;
    }
    if (inTime)
    {
        this.adRefreshTimer = setTimeoutObject(this, "refreshAds", inTime);
    }
    else
    {
        this.adRefreshTimer = setTimeoutObject(this, "refreshAds", this.maxRefreshInterval);
    }
}