// ****************************************************************************
//
// Copyright ?2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************

function component(_gadget)
{
    this.authPanelReady = false;
    this.lastLoadedScreenName = null;
    this.authPaneFirstStart = true;
    this.timer = null;
    this.loadingFromStorage = false;
}

component.prototype.setScreenNameFocused = function()
{
    this.scene.focusedBox = this._gadget.getPartById("screenNameInput");
}

component.prototype.getScreenName = function()
{
    return this._gadget.getPartById("screenNameInput").getAttribute("value");
}

component.prototype.getPassword = function()
{
    return this._gadget.getPartById("passwordInput").getAttribute("value");
}

component.prototype.getPasscode = function()
{
    return this._gadget.getPartById("passcodeInput").getAttribute("value");
}

component.prototype.getCaptcha = function()
{
    return this._gadget.getPartById("captchaInput").getAttribute("value");
}

component.prototype.getSigninPresence = function()
{
    return this._gadget.getPartById("signInInvisibleButton").toggled ? "invisible" : "online"
}

component.prototype.saveToStorage = function()
{
    if (!this.loadingFromStorage)
    {
        this.updateStorage();
    }
}

component.prototype.savePasswordToggled = function()
{
    var autoSignOn = this._gadget.getPartById("autoSignInButton");
    if (!this._gadget.getPartById("savePasswordButton").toggled)
    {
        autoSignOn.toggled = false;
    }
}

component.prototype.setupLoginScreenFromScreenname = function(inScreenname)
{
    if (inScreenname == this.lastLoadedScreenName)
    {
        if (!this._gadget.getPartById("savePasswordButton").toggled && this.getPassword())
        {
            this._gadget.getPartById("passwordInput").setAttribute("value", "");
        }
    }
    else
    {
        this.loadingFromStorage = true;
        this.lastLoadedScreenName = inScreenname;
        try
        {
            var kvs = new keyValueStorage();
            var storeName = this.scene.prefObj.prefsGetStoreForIdentity(inScreenname);
            kvs.open(storeName);
            if (kvs.getAll().allKeys().count > 0)
            {
                var screenName = kvs.getValue(this.scene.prefObj.screenNameKey, inScreenname);

                var protectedPassword = kvs.getValue(this.scene.prefObj.passwordKey, this.scene.prefObj.passwordDefaultValue);
                var password = "";
                if (protectedPassword)
                {
                    try
                    {
                        password = shell.app.unprotectString(protectedPassword);
                    }
                    catch (e) { }
                }

                var isSavePassword;
                var isAutoSignIn
                if (password)
                {
                    isSavePassword = true;
                    isAutoSignIn = kvs.getValue(this.scene.prefObj.autoSignInKey, this.scene.prefObj.autoSignInDefaultValue);
                    this._gadget.getPartById("autoSignInButton").toggled = isAutoSignIn;
                }
                else
                {
                    isSavePassword = false;
                }
                this._gadget.getPartById("savePasswordButton").toggled = isSavePassword;

                var isSignInInvisible = kvs.getValue(this.scene.prefObj.signInInvisibleKey, this.scene.prefObj.signInInvisibleDefaultValue);
                if (this.getScreenName() != screenName)
                {
                    this._gadget.getPartById("screenNameInput").setAttribute("value", screenName);
                }

                this._gadget.getPartById("passwordInput").setAttribute("value", password);
                this._gadget.getPartById("passwordInput").setAttribute("password", true);
                this._gadget.getPartById("signInInvisibleButton").toggled = isSignInInvisible;
            }
            kvs.close();
            kvs = null;
            this.loadingFromStorage = false;
        }
        catch (err)
        {
        }
    }
}

component.prototype.screenNameModified = function()
{
    if (this.authPanelReady)
    {
        var screenNameInput = this._gadget.getPartById("screenNameInput");
        var screenName = screenNameInput.getAttribute("value");
        if (screenName)
        {
            this.setupLoginScreenFromScreenname(screenName);
        }
        else
        {
            this.reset();
        }
        
        this.updateSignInbuttonStatus();
    }
}

component.prototype.reset = function()
{
    this._gadget.getPartById("passwordInput").setAttribute("value", "");
    var savePasswordButton = this._gadget.getPartById("savePasswordButton");
    if (savePasswordButton.toggled != true)
    {
        savePasswordButton.toggled = true;
    }

    var autoSignInButton = this._gadget.getPartById("autoSignInButton");
    if (autoSignInButton.toggled != true)
    {
        autoSignInButton.toggled = true;
    }

    var signInInvisibleButton = this._gadget.getPartById("signInInvisibleButton");
    if (signInInvisibleButton.toggled != false)
    {
        signInInvisibleButton.toggled = false;
    }
}

component.prototype.showLoginScreen = function()
{
    this.setPanelIndex(0);
    var statusBox = this.scene.getBoxById("loginStatusBox");
    statusBox.collapsed = true;
    try
    {
        //** get last used screenname
        // NOTE IMPORTANT: normalise screennames before storage operations on them
        var kvs = new keyValueStorage();
        var storeName = this.scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        var lastSN = kvs.getValue(this.scene.prefObj.previousScreenNameKey, "");
        kvs.close();
        kvs = null;
        //** retrieve and set settings for user (if available)
        this.setupLoginScreenFromScreenname(lastSN);
    }
    catch(err)
    {
    }

    // Place focus on the screenname input field unless the screenname input field is populated and password input field is empty.
    // ...Unless there was an authentication error, in which case set the focus onto the password field
    // (Currently only checks for the presence of the signInErrorLabel and assumes a failure occurred if it's present)
    var focusedBox = this._gadget.getPartById("screenNameInput");
    if ( (this.getScreenName() && !this.getPassword()) || (this._gadget.getAttribute("showSigninErrorLabel") == "true" ) )
    {
        focusedBox = this._gadget.getPartById("passwordInput");
    }
    this.updateSignInbuttonStatus();

    if (!this.authPaneFirstStart)
    {
        propagateEventForBox(this._gadget, "menuChanged");
    }
    else
    {
        this.scene.getBoxById("startupBox").collapsed = true;
        this.authPaneFirstStart = false;
        if (this._gadget.getPartById("autoSignInButton").toggled && this.getScreenName() && this.getPassword())
        {
            OverrideSoftkeyMenus(this.scene, "", "IDM_NOIMPL", this.scene.rootBox.getAttribute("aim.cancel"), "IDM_AUTH_CANCEL");
            statusBox.collapsed = false;
            this.authSignOn();
            focusedBox = null;
        }
    }
    if (focusedBox)
    {
        this.authPanelReady = true;
        this.scene.focusedBox = focusedBox;
    }
}

component.prototype.getIsAuthPaneFirstStart = function()
{
    return this.authPaneFirstStart;
}

component.prototype.showLoginStatusScreen = function()
{
    var loginStatusBox = this.scene.getBoxById("loginStatusBox");
    if (loginStatusBox.collapsed)
    {
        this.scene.focusedBox = null;
        
        // Set up for our transition
        var transitionBox = this.scene.rootBox;
        takeSnapshot(transitionBox);

        // Set the softkey menus with our two buttons
        OverrideSoftkeyMenus(this.scene, "", "IDM_NOIMPL", this.scene.rootBox.getAttribute("aim.cancel"), "IDM_AUTH_CANCEL");
        loginStatusBox.collapsed = false;
        doTransition(transitionBox, "blur", 800, "ease");
    }
}

component.prototype.showCaptchaScreen = function(inStr, inURL)
{
    this.scene.getBoxById("loginStatusBox").collapsed = true;
    this.setPanelIndex(2);
    var captchaInput = this._gadget.getPartById("captchaInput");
    captchaInput.setAttribute("value", "");
    
    var captchaImage = this._gadget.getPartById("captchaImage");
    captchaImage.setAttribute("src", inURL);
    
    var captchaLabel = this._gadget.getPartById("captchaLabel");
    captchaLabel.setAttribute("value", inStr);
    
    var windowBox = this.scene.rootBox;
    OverrideSoftkeyMenus(this.scene, windowBox.getAttribute("signon.message.buttonText"), "IDM_AUTH_CONTINUE", windowBox.getAttribute("aim.cancel"), "IDM_AUTH_CANCEL");
    this.scene.focusedBox = captchaInput;
}

component.prototype.showPasscodeScreen = function()
{
    this.scene.getBoxById("loginStatusBox").collapsed = true;
    this.setPanelIndex(1);
    var passcodeBox = this._gadget.getPartById("passcodeInput");
    passcodeBox.setAttribute("value", "");
    passcodeBox.inputMode = 7;
    
    var windowBox = this.scene.rootBox;
    OverrideSoftkeyMenus(this.scene, windowBox.getAttribute("signon.message.buttonText"), "IDM_AUTH_CONTINUE", windowBox.getAttribute("aim.cancel"), "IDM_AUTH_CANCEL");
    this.scene.focusedBox = passcodeBox;
}

component.prototype.onLoginError = function(msg)
{
    this._gadget.getPartById("errorLabel").setAttribute("title", msg);
    this._gadget.setShowSigninErrorLabel(true);
    this._gadget.showLoginScreen();
}

component.prototype.clearStorageForCurrentUser = function()
{
    var screenName = this._gadget.getScreenName();
    try
    {
        var kvs = new keyValueStorage();
        // store screenname:password in "knownIdentities" storage
        var storeName = this.scene.prefObj.prefsGetKnownIdentitiesStore();
        kvs.open( storeName );
        kvs.setValue( screenName.toLowerCase(), "" );
        kvs.close();
        
        // store user settings in <screenName> storage
        storeName = this.scene.prefObj.prefsGetStoreForIdentity(screenName);
        kvs.open( storeName );
        
        kvs.removeKey(this.scene.prefObj.screenNameKey);
        kvs.removeKey(this.scene.prefObj.passwordKey);
        kvs.removeKey(this.scene.prefObj.autoSignInKey);
        kvs.removeKey(this.scene.prefObj.signInInvisibleKey);
        kvs.close();
        kvs = null;
    }
    catch(err)
    {
    }
}

component.prototype.updateStorage = function()
{
    var screenName = this._gadget.getScreenName();
    var password = this._gadget.getPassword();
    var autoSignIn = this._gadget.getPartById("autoSignInButton").getAttribute("toggled");
    var signInInvisible = this._gadget.getPartById("signInInvisibleButton").getAttribute("toggled");

    try
    {
        var kvs = new keyValueStorage();
        // store screenname:password in "knownIdentities" storage
        var storeName = this.scene.prefObj.prefsGetKnownIdentitiesStore();
        kvs.open( storeName );
        kvs.setValue( screenName.toLowerCase(), "" );
        kvs.close();
        // store user settings in <screenName> storage
        storeName = this.scene.prefObj.prefsGetStoreForIdentity(screenName);
        kvs.open( storeName );
        kvs.setValue( this.scene.prefObj.screenNameKey,  screenName );
        
        if (this._gadget.getPartById("savePasswordButton").toggled)
        {
            try
            {
                var encPwd = shell.app.protectString(password);
            }
            catch(e){}
            kvs.setValue( this.scene.prefObj.passwordKey, encPwd );
        }
        else
        {
            //clear any password that was stored there
            kvs.removeKey( this.scene.prefObj.passwordKey);
        }
        
        kvs.setValue( this.scene.prefObj.autoSignInKey , autoSignIn);
        kvs.setValue( this.scene.prefObj.signInInvisibleKey , signInInvisible);
        kvs.close();
        // store screenname as last used screenname
        storeName = this.scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        kvs.setValue( this.scene.prefObj.previousScreenNameKey, screenName.toLowerCase() );
        kvs.close();
        kvs = null;
    }
    catch(err)
    {
    }
}

component.prototype.authSignOn = function()
{
    if (!this._gadget.getPartById("signInButton").disabled)
    {
        this.authPanelReady = false;
        this.scene.focusedBox = null;
        this.updateStorage();
        
        var screenName = this._gadget.getScreenName();
        var password = this._gadget.getPassword();
        this.setShowSigninErrorLabel(false);
        var event = shell.factory.createEvent(this._gadget);
        if (event)
        {	    
            this.showLoginStatusScreen();
            event.type = "authSignOn";
            event.screenname = screenName;
            event.password = password;
            event.presence = this._gadget.getSigninPresence();
            this._gadget.dispatchEvent(event);
        }
    }
}



component.prototype.setShowSigninErrorLabel = function(show)
{
    this._gadget.setAttribute("showSigninErrorLabel", show ? "true" : "false");
}

component.prototype.getPanelIndex = function()
{
    return this._gadget.getPartById("authStepPanels").getAttribute("topIndex");
}

component.prototype.setPanelIndex = function(index)
{
    return this._gadget.getPartById("authStepPanels").setAttribute("topIndex", index.toString());
}

component.prototype.submitPasscode = function()
{
    if (this.getPanelIndex() == "1")
    {
        var event = shell.factory.createEvent(this._gadget);
	    if (event)
	    {	    
	        this.showLoginStatusScreen();
		    event.type = "authSubmitPasscode";
		    event.passcode = this._gadget.getPasscode();;
		    this._gadget.dispatchEvent(event);
	    }
	}
}

component.prototype.submitCaptcha = function()
{
    if (this.getPanelIndex() == "2")
    {
        var event = shell.factory.createEvent(this._gadget);
	    if (event)
	    {	    
	        this.showLoginStatusScreen();
		    event.type = "authSubmitCaptcha";
		    event.captcha = this._gadget.getCaptcha();
		    this._gadget.dispatchEvent(event);
	    }
	}
}

component.prototype.authSignOnCancel = function()
{
    this.setShowSigninErrorLabel(false);
    this.showLoginScreen();
}

component.prototype.quitApplication = function()
{
    this.scene.close();
}

component.prototype.passwordFocusChanged = function()
{
    var passwordInput = this._gadget.getPartById("passwordInput");
    if (passwordInput.focused)
    {
        var password = passwordInput.getAttribute("value");
        if (!password)
        {
            passwordInput.setAttribute("password", false);
        }
    }
    else
    {
        if (this.timer)
        {
            this.timer.cancel();
            this.timer = null;
        }
        passwordInput.setAttribute("password", true);
    }
}

component.prototype.passwordModified = function()
{
    if (this.authPanelReady)
    {
        if (this.timer)
        {
            this.timer.cancel();
            this.timer = null;
        }
        var passwordInput = this._gadget.getPartById("passwordInput");
        var password = passwordInput.getAttribute("value");
        if (password)
        {
            passwordInput.setAttribute("password", false);
            this.timer = setTimeoutObject(this._gadget, "maskPassword", 2000);
        }
        else
        {
            passwordInput.setAttribute("password", false);
        }
        
        this.updateSignInbuttonStatus();
    }
}

component.prototype.updateSignInbuttonStatus = function()
{
    var disableSignIn = (!this.getScreenName() || !this.getPassword()) ? true : false;
    this._gadget.getPartById("signInButton").disabled = disableSignIn;
}


component.prototype.maskPassword = function()
{
    this._gadget.getPartById("passwordInput").setAttribute("password", true);
}

component.prototype.unmaskPassword = function()
{
    this._gadget.getPartById("passwordInput").setAttribute("password", false);
}

component.prototype.getAccount = function()
{
    try
    {
        appUtils.shellExecute("http://wap.aol.com/auth/menu.do?id=6500");
    }
    catch(e){}
}

