// ****************************************************************************
//
// Copyright ?2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************


// ****************************************************************************
// Buddy object
// ****************************************************************************

var ePendingActions = { kNone:0, kAdd:1, kDelete:2, kRename:3 };
var eFlags          = { kNone:1, kNewMessage:2};

var GROUPID_OFFLINE_BUDDIES = "Offline Buddies";
var GROUPID_RECENT_BUDDIES = "Recent Buddies";

function objBuddyListBuddy(aimId, groupId)
{
    this.displayId = this.aimId = canonicalizeScreenName(aimId);
    this.groupId = groupId;
    this.pendingAction = ePendingActions.kNone;
    this.flags = eFlags.kNone;
    this.state = "offline";
    this.statusMsg = "";
    this.buddyIcon = "";
    this.idleTime = 0;
}

objBuddyListBuddy.prototype.updateBuddyFromObj = function(buddyObj)
{
    if (buddyObj)
    {
        this.updateBuddy(buddyObj.displayId,
                            buddyObj.state,
                            buddyObj.statusMsg,
                            buddyObj.buddyIcon,
                            buddyObj.idleTime,
                            buddyObj.flags);
    }
}

objBuddyListBuddy.prototype.updateBuddy = function(displayId, state, statusMsg, buddyIcon, idleTime, flags)
{
    if (displayId && this.displayId != displayId)
    {
        this.displayId = displayId;
    }
    if (state && this.state != state)
    {
        this.state = state;
    }
    if (this.statusMsg != statusMsg)
    {
        if (statusMsg && statusMsg.indexOf("%") > -1)
        {
            if (statusMsg.indexOf("%n") > -1)
            {
                statusMsg = statusMsg.replace(/%n/ig, scene.AIMobj.getLoginId());
            }
            
            var date;
            if (statusMsg.indexOf("%d") > -1)
            {
                date = new Date()
                var dateStr = date.getYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
                statusMsg = statusMsg.replace(/%d/ig, dateStr);
            }
            if (statusMsg.indexOf("%t") > -1)
            {
                if (!date)
                {
                    date = new Date();
                }
                var hours = date.getHours();
                var mins = date.getMinutes();
                var timestamp = ((hours > 12) ? (hours-12) : hours) + ":" + (mins < 10 ? "0"+mins : mins);
                statusMsg = statusMsg.replace(/%t/ig, timestamp);
            }
        }
    
        this.statusMsg = statusMsg ? statusMsg : "";
    }
    if (buddyIcon && this.buddyIcon != buddyIcon)
    {
        this.buddyIcon = buddyIcon;
    }
    if (idleTime && this.idleTime != idleTime)
    {
        this.idleTime = idleTime;
    }
    if (flags && this.flags != flags)
    {
        this.flags = flags;
    }
}

// ****************************************************************************
// Group object
// ****************************************************************************

function objBuddyListGroup(groupObj)
{
    this.visibleBuddies = new liveArray;
    this.invisibleBuddies = new xArray;
    this.pendingAction = ePendingActions.kNone;
    
    this.totalBuddies = 0;
    this.onlineBuddies = 0;

    this.collapsed = 0;
        
    if (groupObj)
    {
        // Server returns collapsed=1 if the group is collapsed.
        if (groupObj.collapsed)
        {
            this.collapsed = 1;
        }
        this.name = groupObj.name;
       
        // Go through this group's buddies and create each
        var buddies = groupObj.buddies;
        if (buddies)
        {
            for (var b=0; b < buddies.length; b++)
            {
                var buddy = buddies[b];
                
                if (!this.hasBuddy(buddy.aimId))
                {
                    var newBuddy = new objBuddyListBuddy(buddy.aimId, this.name);
                    if (newBuddy)
                    {
                        if (buddy.state == "mobile")
                        {
                            buddyIcon = "#buddyList.placeHolderText";
                        }
                        
                        if (buddy.friendly)
                        {
                            buddy.displayId = buddy.friendly;
                        }

                        newBuddy.updateBuddyFromObj(buddy);
                        this.addBuddyToGroup(newBuddy);
                            
                        this.totalBuddies++;
                    }
                }
            }
        }
        
        this.updateOnlineCount();
    }
}

objBuddyListGroup.prototype.hasBuddy =  function(aimId)
{
    if(this.visibleBuddies.indexOfKey("aimId", aimId) != -1 || this.invisibleBuddies.indexOfKey("aimId", aimId) != -1)
        return true;
    else    
        return false;

}

// ----------------------------------------------------------------------------

objBuddyListGroup.prototype.getOnlineBuddyCount = function()
{
    return this.visibleBuddies.count;
}

objBuddyListGroup.prototype.getTotalBuddyCount = function()
{
    return this.visibleBuddies.count + this.invisibleBuddies.count;
}

objBuddyListGroup.prototype.updateOnlineCount = function()
{
    var isUpdated = false;
    if (this.onlineBuddies != this.visibleBuddies.count)
    {
        this.onlineBuddies = this.visibleBuddies.count;
        isUpdated = true;
    }
    return isUpdated;
}

// ----------------------------------------------------------------------------
objBuddyListGroup.prototype.addBuddyToGroup = function(buddyObj)
{
    if(buddyObj.state != "offline" && buddyObj.state != "blocked")
        this.visibleBuddies.addValue(buddyObj);
    else
        this.invisibleBuddies.addValue(buddyObj);
}

objBuddyListGroup.prototype.moveBuddy = function(buddyData, priorIndex, isMovingOffline)
{
    if(isMovingOffline)
    {
        this.visibleBuddies.removeValueAtIndex(priorIndex);
        this.invisibleBuddies.addValue(buddyData);
    }
    else
    {
        this.invisibleBuddies.removeValueAtIndex(priorIndex);
        this.visibleBuddies.addValue(buddyData);
    }
} 

objBuddyListGroup.prototype.updateBuddy = function(buddy)
{
    var index = this.visibleBuddies.indexOfKey("aimId", buddy.aimId);
    if(index != -1)
    {
        if((this.name != GROUPID_OFFLINE_BUDDIES && (buddy.state == "offline" || buddy.state == "blocked"))
            || (this.name == GROUPID_OFFLINE_BUDDIES && buddy.state != "offline" && buddy.state != "blocked"))
        {
            this.moveBuddy(buddy, index, true);
        }
        else 
        {
            this.visibleBuddies.replaceValueAtIndex(index, buddy);
        }        
    }
    else
    {
        index = this.invisibleBuddies.indexOfKey("aimId", buddy.aimId);
        if(index != -1)
        {
            if((this.name != GROUPID_OFFLINE_BUDDIES && buddy.state != "offline" && buddy.state != "blocked")
                || (this.name == GROUPID_OFFLINE_BUDDIES && (buddy.state == "offline" || buddy.state == "blocked")))
            {
                this.moveBuddy(buddy, index, false);
            }
            else
            {
                this.invisibleBuddies.replaceValueAtIndex(index, buddy);
            }
        }
    }
}

objBuddyListGroup.prototype.deleteBuddyFromGroup = function(buddy)
{
    var index = this.visibleBuddies.indexOfKey("aimId", buddy.aimId);
    if(index != -1)
    {
        this.visibleBuddies.removeValueAtIndex(index, buddy);
        return 0;
    }
    
    index = this.invisibleBuddies.indexOfKey("aimId", buddy.aimId);
    if(index != -1)
    {
        this.invisibleBuddies.removeValueAtIndex(index);
    }

}

objBuddyListGroup.prototype.addBuddy = function(aimId, isPending)
{
    
    // First see if this buddy already exists in this group
    var buddy = this.getBuddy(aimId);
    
    if (buddy)
    {
        // Just update the existing buddy item
        buddy.pendingAction = isPending ? ePendingActions.kAdd : ePendingActions.kNone;
        this.updateBuddy(buddy);
    }
    else
    {
        // This buddy doesn't exist in this group yet - create it and add it to the group
        buddy = new objBuddyListBuddy(aimId, this.name);
        if (buddy)
        {
            buddy.pendingAction = isPending ? ePendingActions.kAdd : ePendingActions.kNone;
            this.addBuddyToGroup(buddy);
            this.totalBuddies++;
        }
    }
    return buddy;
}

// ----------------------------------------------------------------------------

objBuddyListGroup.prototype.deleteBuddy = function(aimId, isPending)
{
    var buddy;
    if (aimId)
    {
        buddy = this.getBuddy(canonicalizeScreenName(aimId)); 
        if (buddy)
        {
            if (isPending)
            {
                // Just mark the item as pending deletion
                buddy.pendingAction = ePendingActions.kDelete;
                this.updateBuddy(buddy);
            }
            else
            {
                // Remove the item
                this.deleteBuddyFromGroup(buddy);
                this.onlineBuddies = this.visibleBuddies.count;
                this.totalBuddies--;
            }
        }
    }
    return buddy;
}

// ----------------------------------------------------------------------------

objBuddyListGroup.prototype.getBuddy = function(aimId)
{
    if (aimId)
    {
        var buddy = null;
        var index = this.visibleBuddies.indexOfKey("aimId", aimId);
        if(index != -1)
        {
            buddy = this.visibleBuddies.valueAtIndex(index);
            return buddy;
        }
        
        index = this.invisibleBuddies.indexOfKey("aimId", aimId);
        if(index != -1)
            buddy = this.invisibleBuddies.valueAtIndex(this.invisibleBuddies.indexOfKey("aimId", aimId));
            
        return buddy;
        
    }
    return null;
}

objBuddyListGroup.prototype.removeAllBuddies = function()
{
    this.visibleBuddies.removeAllValues();
    this.invisibleBuddies.removeAllValues();
    this.totalBuddies = 0;
    this.onlineBuddies = 0;
}
// ****************************************************************************
// Buddy List ad
// ****************************************************************************

function objBuddyListAd()
{

    //this.TSMadspaceid = "75084237";
    this.TSMadspaceid = "75084176";
    this.TSMpublisherid = "75084";
    this.TSMadsize="167x30";
}

// ****************************************************************************
// Buddy List object
// ****************************************************************************

function objBuddyList()
{
    this.groups = new liveArray;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.buildOfflineBuddyGroup = function()
{
    var offlineGroup = this.getOfflineBuddyGroup();
    if (offlineGroup)
    {
        offlineGroup.removeAllBuddies();
        
        // Update the offline group
        var enumGroup = this.groups.getEnumerator();
        while (enumGroup.moveNext())
        {
            var group = enumGroup.current;
            if (group == offlineGroup)
                continue;
            
            if (group.invisibleBuddies)
            {
                var enumBuddies = group.invisibleBuddies.getEnumerator();
                while (enumBuddies.moveNext())
                {
                    var buddy = enumBuddies.current;
                    if (!offlineGroup.hasBuddy(buddy.aimId))
                    {
                        var offlineBuddy = new objBuddyListBuddy(buddy.aimId, GROUPID_OFFLINE_BUDDIES);
                        offlineBuddy.updateBuddyFromObj(buddy);
                        offlineGroup.visibleBuddies.addValue(offlineBuddy);
                        offlineGroup.totalBuddies++;
                    }
                }
            }
            
            if(group.visibleBuddies)
            {
                var enumBuddies = group.visibleBuddies.getEnumerator();
                while (enumBuddies.moveNext())
                {
                    var buddy = enumBuddies.current;
                    if (!offlineGroup.hasBuddy(buddy.aimId))
                    {
                        var offlineBuddy = new objBuddyListBuddy(buddy.aimId, GROUPID_OFFLINE_BUDDIES);
                        offlineBuddy.updateBuddyFromObj(buddy);
                        offlineGroup.invisibleBuddies.addValue(offlineBuddy);
                        offlineGroup.totalBuddies++;
                    }
                }
            }
        }
        
        offlineGroup.onlineBuddies = offlineGroup.visibleBuddies.count;
        
        // Update the group
        this.groups.replaceValueAtIndex(this.groups.indexOfKey("name", GROUPID_OFFLINE_BUDDIES), offlineGroup);
    }        
}

objBuddyList.prototype.getRecentBuddyGroup = function()
{
    return this.getGroupByName(GROUPID_RECENT_BUDDIES);
}

objBuddyList.prototype.getOfflineBuddyGroup = function()
{
    return this.getGroupByName(GROUPID_OFFLINE_BUDDIES);
}

objBuddyList.prototype.getGroupIndexByName = function(name)
{
    return this.groups.indexOfKey("name", name);
}

objBuddyList.prototype.getGroupByName = function(name)
{
    var index = this.getGroupIndexByName(name);
    if (index != -1)
    {
        return this.groups.getValue(index);
    }
    return null;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.addBuddyToOfflineGroup = function(buddyId)
{
    var offlineGroup;
    var offlineIdx = this.getGroupIndexByName(GROUPID_OFFLINE_BUDDIES);
    if (offlineIdx != -1)
    {
        offlineGroup = this.groups.getValue(offlineIdx);
    }
    if (offlineGroup)
    {
        // First make sure it's not already in the group
        if (!offlineGroup.hasBuddy(buddyId))
        {
            var buddy = new objBuddyListBuddy(buddyId, GROUPID_OFFLINE_BUDDIES);
            buddy.state = "offline";
            
            offlineGroup.visibleBuddies.addValue(buddy);
            offlineGroup.onlineBuddies = offlineGroup.visibleBuddies.count;
            offlineGroup.totalBuddies++;
            
            this.groups.replaceValueAtIndex(offlineIdx, offlineGroup);
        }
    }
}


// ----------------------------------------------------------------------------

objBuddyList.prototype.insertGroup = function(group)
{
    if (group)
    {
        var index = this.groups.indexOfKey("name", GROUPID_OFFLINE_BUDDIES);
        if (index != -1)
        {
            // Insert the new group before the offline buddies group
            this.groups.insertValueAtIndex(group, index);
        }
        else
        {
            // No offline buddies group - just add the group normally
            this.groups.addValue(group);
        }
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.clear = function()
{
    // Delete the current data
    this.groups.removeAllValues();
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.addBuddy = function(buddyId, groupId, isPending)
{
    var index = this.groups.indexOfKey("name", groupId);
    if (index == -1 && groupId == GROUPID_RECENT_BUDDIES)
    {
        var groupObject = new Object();
        groupObject.name = GROUPID_RECENT_BUDDIES;
        this.insertGroup(new objBuddyListGroup(groupObject));
        index = this.groups.indexOfKey("name", GROUPID_RECENT_BUDDIES);
    }
    if (index != -1)
    {
        var group = this.groups.getValue(index);
        if (group)
        {
            var buddy = group.addBuddy(buddyId, isPending);
            this.groups.replaceValueAtIndex(index, group);
            
            this.notifyPropertyChange(buddy, "add");
        }
    }
    // else that group doesn't exist
    
    // Add every buddy to the offline group
    this.addBuddyToOfflineGroup(buddyId);
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.addGroup = function(groupId, isPending)
{
    var group = null;
    
    // First see if this buddy already exists in this group
    var index = this.groups.indexOfKey("name", groupId);
    if (index != -1)
    {
        group = this.groups.getValue(index);
    }
    
    if (group)
    {
        // Found an existing group with this id - just update the existing group
        group.pendingAction = isPending ? ePendingActions.kAdd : ePendingActions.kNone;
        this.groups.replaceValueAtIndex(index, group);
    }
    else
    {
        // Create a new group and add it to our group list
        var group = new objBuddyListGroup(null);
        if (group)
        {
            group.name = groupId;
            group.pendingAction = isPending ? ePendingActions.kAdd : ePendingActions.kNone;
            this.insertGroup(group);
        }
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.renameGroup = function(oldGroupId, newGroupId, isPending)
{
    if (oldGroupId && newGroupId && (oldGroupId != GROUPID_OFFLINE_BUDDIES))
    {
        var index = this.groups.indexOfKey("name", isPending ? oldGroupId : newGroupId);
        if (index != -1)
        {
            var group = this.groups.getValue(index);

            group.name = newGroupId;
            group.pendingAction = isPending ? ePendingActions.kRename : ePendingActions.kNone;
            
            if (!isPending)
            {
                // Unfortunately now we have to go through every single buddy in this group and change the groupId
                var buddies = group.visibleBuddies;
                
                var buddyCount = buddies.count;
                
                for (i=0; i<buddyCount; i++)
                {
                    var buddy = buddies.valueAtIndex(i);
                    buddy.groupId = newGroupId;
                    buddies.replaceValueAtIndex(i, buddy);
                }
                
                buddies = group.invisibleBuddies;                
                buddyCount = buddies.count;
                
                for (i=0; i<buddyCount; i++)
                {
                    var buddy = buddies.valueAtIndex(i);
                    buddy.groupId = newGroupId;
                    buddies.replaceValueAtIndex(i, buddy);
                }

            }
            
            
            // Replace it
            this.groups.replaceValueAtIndex(index, group);
        }
        // else that group does not exit
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.updateBuddiesFromObj = function(buddyData)
{
    if (buddyData)
    {
        // Go through each group, looking for this buddy
        var aimId = buddyData.aimId;
        var tempBuddy;
        var numGroups = this.groups.count;
        for (groupIndex=0; groupIndex < numGroups; groupIndex++)
        {
            var group = this.groups.valueAtIndex(groupIndex);
            if (group && group.visibleBuddies)
            {
                // Look for the buddyId in this group   
                var buddy = group.getBuddy(aimId);
                if (buddy)
                {
                    // Update the buddy properties
                    buddy.updateBuddyFromObj(buddyData);
                    
                    // Replace it
                    group.updateBuddy(buddy);
                    
                    // Update the group's online count
                    if (group.updateOnlineCount())
                    {
                        //don't update the binding unless the user changes state
                        //to switch between offline and online group
                        this.groups.replaceValueAtIndex(groupIndex, group);
                    }
                    if (!tempBuddy)
                    {
                        tempBuddy = buddy;
                    }
                }
            }
        }
        this.notifyPropertyChange(tempBuddy, "update");
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.setBuddyPendingAction = function(buddyId, groupId, pendingAction)
{
    if (buddyId && groupId)
    {
        var groups = this.groups;
        
        var groupIndex = groups.indexOfKey("name", groupId);
        if (groupIndex != -1)
        {
            var group = groups.getValue(groupIndex);
            if (group)
            {
                if (group.visibleBuddies)
                {
                    // Look for the buddyId in this group   
                    var buddy = group.getBuddy(buddyId);
                    if (buddy != null)
                    {
                        // Found it - get the buddy object
                        // Update the buddy properties
                            buddy.pendingAction = pendingAction;
                            
                            // Replace it
                            group.updateBuddy(buddy);
                    }
                    
                    groups.replaceValueAtIndex(groupIndex, group);
                }
            }
        }
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.setGroupPendingAction = function(groupId, pendingAction)
{
    if (groupId)
    {
        var groups = this.groups;
        
        var groupIndex = groups.indexOfKey("name", groupId);
        if (groupIndex != -1)
        {
            var group = groups.getValue(groupIndex);
            if (group)
            {
                group.pendingAction = pendingAction;
                group.replaceValueAtIndex(groupIndex, group);
            }
        }
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.getGroup = function(groupId)
{
    if (groupId)
    {
        var groups = this.groups;
        
        var index = groups.indexOfKey("name", groupId);
        if (index != -1)
        {
            var group = groups.getValue(index);
            if (group.visibleBuddies)
                return group;
        }
    }
    return null;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.deleteBuddy = function(buddyId, groupId, isPending)
{
    if (groupId && buddyId)
    {
        var groups = this.groups;
        var buddy;
        if (groupId == GROUPID_OFFLINE_BUDDIES)
        {
            // If the buddy is being deleted from the offline buddies group,
            // we need to go and remove that buddy from every buddy list it appears in
            var numGroups = this.groups.count;
            for (groupIndex=0; groupIndex<numGroups; groupIndex++)
            {
                var group = this.groups.valueAtIndex(groupIndex);
                if (group && group.visibleBuddies)
                {
                    buddy = group.deleteBuddy(buddyId, isPending);
                    groups.replaceValueAtIndex(groupIndex, group);
                }                
            }
        }
        else
        {
            // Remove the specified buddy from the specified group
            var index = groups.indexOfKey("name", groupId);
            if (index != -1)
            {
                var group = groups.getValue(index);
                if (group && group.visibleBuddies)
                {
                    buddy = group.deleteBuddy(buddyId, isPending);
                    groups.replaceValueAtIndex(index, group);
                }
            }
        }
        if (buddy)
        {
            this.notifyPropertyChange(buddy, "delete");
            if (!isPending)
            {
                var list = this.getGroupListForBuddy(buddy);
                if (list && list.length <= 0)
                {
                    var offlineGroup;
                    var offlineIdx = this.getGroupIndexByName(GROUPID_OFFLINE_BUDDIES);
                    if (offlineIdx != -1)
                    {
                        offlineGroup = this.groups.getValue(offlineIdx);
                    }
                    if (offlineGroup)
                    {
                        offlineGroup.deleteBuddy(buddyId, false);
                        groups.replaceValueAtIndex(offlineIdx, offlineGroup);
                    }
                }
            }
        }
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.deleteGroup = function(groupId, isPending)
{
    if (groupId && (groupId != GROUPID_OFFLINE_BUDDIES))
    {
        var groups = this.groups;
        
        var index = groups.indexOfKey("name", groupId);
        if (index != -1)
        {
            if (isPending)
            {
                // Mark the group as pending deletion
                var group = groups.getValue(index);
                group.pendingAction = ePendingActions.kDelete;
                groups.replaceValueAtIndex(index, group);
            }
            else
            {
                // Remove the group
                groups.removeValueAtIndex(index);
            }
        }
    }
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.getBuddy = function(buddyId, groupId)
{
    if (buddyId)
    {
        var group = this.getGroup(groupId);
        if (group)
        {
            return group.getBuddy(buddyId);
        }
    }
    
    return null;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.getBuddyInfo = function(buddyId)
{
    // Go through all groups looking for this buddy
    
    var numGroups = this.groups.count;
    for (groupIndex=0; groupIndex<numGroups; groupIndex++)
    {
        var group = this.groups.valueAtIndex(groupIndex);
        if (group.visibleBuddies)
        {        
            // Look for the buddyId in this group   
            var buddy = group.getBuddy(buddyId);
            if (buddy != null)
            {
                // Found it
                return buddy;
            }
        }
    }
    
    return null;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.getGroupList = function()
{
    var groupList = new Array();
    if (groupList)
    {
        var enumGroup = this.groups.getEnumerator();
        while (enumGroup.moveNext())
        {
            var group = enumGroup.current;
            if (group.visibleBuddies && (group.name != GROUPID_OFFLINE_BUDDIES && group.name != GROUPID_RECENT_BUDDIES))
            {
                groupList.push(group.name);
            }
        }
    }
    
    return groupList;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.getGroupListForBuddy = function(buddyId)
{
    var groupList = new Array();
    if (groupList)
    {
        var enumGroup = this.groups.getEnumerator();
        while (enumGroup.moveNext())
        {
            var group = enumGroup.current;
            if (group.visibleBuddies && (group.name != GROUPID_OFFLINE_BUDDIES))
            {
                if (group.hasBuddy(buddyId))
                    groupList.push(group.name);
            }
        }
    }
    
    return groupList;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.udpateCollapsedForGroup = function(groupId, collapsed)
{
    var numGroups = this.groups.count;
    for (groupIndex=0; groupIndex<numGroups; groupIndex++)
    {
        var group = this.groups.valueAtIndex(groupIndex);
        if (group.name == groupId)
        {
            group.collapsed = collapsed;
            // Update the group.
            this.groups.replaceValueAtIndex(groupIndex, group);
            return null;
        }
    }
    
    return null;
}

// ----------------------------------------------------------------------------

objBuddyList.prototype.matchBuddiesForStr = function(str)
{
    var matchBuddies;
    var numGroups = this.groups.count;
    for (groupIndex = 0; groupIndex < numGroups; ++groupIndex)
    {
        var group = this.groups.valueAtIndex(groupIndex);
        var visibleBuddies = group.visibleBuddies;
        if (visibleBuddies)
        {
            var count = visibleBuddies.count;
            for (var x = 0; x < count; ++x)
            {
                var buddyObj = visibleBuddies.valueAtIndex(x);
                if (buddyObj && buddyObj.displayId.toLowerCase().indexOf(str) > -1)
                {
                    if (!matchBuddies)
                    {
                        matchBuddies = new liveArray();
                    }
                    if (matchBuddies.indexOfKey("displayId", buddyObj.displayId) == -1)
                    {
                        buddyObj.matchedValue = str;
                        matchBuddies.addValue(buddyObj);
                    }
                }            
            }
        }
    }
    return matchBuddies;
}

// Support for the EEObjectSource protocol
objBuddyList.prototype.listeners = [];

objBuddyList.prototype.addListener = function( object )
{
    this.listeners.push(object);
}

objBuddyList.prototype.removeListener = function( object )
{
    for (i = 0; i < this.listeners.length; i++)
    {
        if (this.listeners[i] == object)
        {
            this.listeners[i] = null;
            this.listeners.splice(i, 1);
            break;
        }
    }
}

objBuddyList.prototype.notifyPropertyChange = function( data, type )
{
    if (this.listeners != null)
    {
        for (i = 0; i < this.listeners.length; i++)
        {
            if (this.listeners[i] != null)
            {
                this.listeners[i].onBuddyListChanged( data, type );
            }
        }
    }
}

// ----------------------------------------------------------------------------
objBuddyList.prototype.getBuddyList = function()
{
    var buddyList = new xArray;

    var enumGroup = this.groups.getEnumerator();
    while (enumGroup.moveNext())
    {
        var group = enumGroup.current;
        var total = group.totalBuddies;
        if (group.visibleBuddies && group.visibleBuddies.count > 0)
        {
            var enumVisBuddies = group.visibleBuddies.getEnumerator();
            while (enumVisBuddies.moveNext())
            {
                var visBuddy = enumVisBuddies.current;
                buddyList.addValue(visBuddy);
            }
        }
    }
    
    return buddyList;
}

