// ****************************************************************************
//
// Copyright 2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************
//  clientLogin.js
//
//****************************************************************************

var _gClientLogin = null;

function ClientLogin(inDevId, inClientName, inClientVersion, inCallbackObj)
{
    this.openAuth = new xOpenAuth();
    this.openAuth.devId = inDevId;
    this.openAuth.clientName = inClientName;
    this.openAuth.clientVersion = inClientVersion;
    this.openAuth.listener = this;
    
    this.callbackObj   = inCallbackObj;
    
    // For development in Win32
    try 
    {
        this.openAuth.userAgent = shell.app.userAgent;
    } 
    catch (e) 
    {
        this.openAuth.userAgent = "mobileAIM";
    }
    
    // Keep a global object of us around
    _gClientLogin = this;
}

ClientLogin.prototype.reset = function()
{
    this.openAuth.reset();
} 


ClientLogin.prototype.signOn = function (sn, pw, forceCaptcha)
{    
    this.openAuth.loginId = sn;
    this.openAuth.password = pw;

    // If we want to show a captcha, set the parameter
    if (forceCaptcha)
        this.openAuth.forceRateLimit = true;
        
    this.openAuth.locale = shell.app.locale.toLowerCase();
    
    var d = new Date();
    var timeStamp = d.getTime();
    this.openAuth.requestId = timeStamp;

    this.openAuth.login();
}

ClientLogin.prototype.onAuthLoginResponse = function (authRequest, authResponse)
{
    if(authResponse.httpStatusCode == 200)
    {
        if (authResponse.statusCode == 200)
        {
            // Start an AIM session
            this.callbackObj.clPostSignOn();
        }
        else
        {
            if (authResponse.statusCode == 330)
            {
                if (authResponse.statusDetailCode == 3012 || authResponse.statusDetailCode == 3013)
                {
                    this.callbackObj.clEnterPasscode(authResponse.challengeInfo);
                }
                else if (authResponse.statusDetailCode == 3015)
                {
                    this.callbackObj.clEnterCaptcha(authResponse.challengeInfo, authResponse.captchaUrl);
                }
                else if (authResponse.statusDetailCode == 3011)
                {
                    this.callbackObj.clAuthError();
                }
                else
                {
                    this.callbackObj.clSignOnError("Error "+(authResponse.statusCodeDetail)? authResponse.statusCodeDetail: authResponse.statusCode +": "+authResponse.statusText);
                }
            }
            else 
            {
                this.callbackObj.clAuthError();
            }
        }
    }
    else
    {
        if (authResponse.httpStatusCode > 12000)
        {
            this.callbackObj.clSignOnError("Connectivity error " + authResponse.httpStatusCode);
        }
        else
        {            
            this.callbackObj.clSignOnError("Authentication server error " + authResponse.httpStatusCode);
        }
    }
}
ClientLogin.prototype.onAuthLogoutResponse = function (authRequest, authResponse)
{
    this.openAuth.reset();
}

ClientLogin.prototype.onAuthResponse = function (authRequest, authResponse)
{
    if (authResponse.requestType == 1) //login
    {
        this.onAuthLoginResponse(authRequest, authResponse);
    }
    else if (authResponse.requestType == 5) //logout
    {
        this.onAuthLogoutResponse(authRequest, authResponse);       
    }
}

ClientLogin.prototype.logout = function()
{
    this.openAuth.logout();
}




