// ****************************************************************************
//
// Copyright ?2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************

var max_icons = 20;
var scroll_delay = 100;

function component (_gadget)
{
    var grabPosY;
    var targetYPos;
    var currentYPos;
    var lastYPos;
    var speedY;
    var lastSpeedY;
    var iconHeight;
    var dragging;
    var overscan;
    var numIcons;
    var age;
    var prevActiveBoxIdx;
    this.showTransition = true;
}

component.prototype.setShowTransition = function(show)
{
    this.showTransition = show;
}

//****************************************************************************

component.prototype.onResized = function()
{
    // If we have been resized, update our calculations
    this.onFrame();
}

//****************************************************************************

component.prototype.onBoxCreated = function()
{
    this.overscan = 0;
    this.grabPosY = 0;
    this.speedY = 0;
    this.lastSpeedY = 0;
    this.dragging = false;
    this.numIcons = 0;
    this.age = 0;
    this.prevActiveBoxIdx = -1;
}

//****************************************************************************

component.prototype.onPresented = function()
{
    this.iconHeight = parseInt(this._gadget.getAttribute("iconHeight"));
    if (!(this.iconHeight > 0))
        this.iconHeight = 44;
    
    var slider = this._gadget.getPartById("slider");
    
    var sliderHeight = slider.boxHeight;
    var gadgetHeight = this._gadget.boxHeight;
    
    this.targetYPos = 0;
    this.currentYPos = 0;
    this.lastYPos = 0;
    slider.setStyle("top", "0");
    
    this._gadget.setAttribute("presented", "true");

    // Set the ad
    var convAd = this._gadget.getPartById("convAd");
    if(convAd)
    {
        var imageAd = convAd.getPartById("icon");
        if(imageAd)
            imageAd.TSMretrieveAd();
    }
    
    // Get the initial view looking right
    this.onFrame();
}

//****************************************************************************

component.prototype.onMouseDown = function()
{
    var event = shell.currentEvent;
    if (event)
    {
        var box = event.targetBox;
        if (box)
        {
            var buddyId = box.getAttribute("buddyId");
            if (buddyId && (buddyId != "") && (buddyId != "undefined"))
            {
                // Make the icon look pressed
                var icon = this.findBuddy(buddyId);
                if (icon)
                {
                    icon.focused = true;
                    icon.setAttribute("pressed", "true");
                }
            
                // activate the icon that was clicked on
                this.activateBuddy(buddyId);
                
                // Unpress the icon
                if (icon)
                    icon.setAttribute("pressed", "false");
                
                event.preventDefault();
            }
        }
    }
}

//****************************************************************************

component.prototype.onFrame = function()
{
    if (this.dragging)
        return;
        
    if (this._gadget.getAttribute("presented") == "true")
    {
        var slider = this._gadget.getPartById("slider");
        var sliderHeight = slider.boxHeight;
        
        var track = this._gadget.getPartById("track");
        var trackHeight = track.boxHeight;

        if (sliderHeight <= trackHeight)
        {
            var newPos = 0;
            this.speedY = 0;
            this.currentYPos = newPos;
            this.targetYPos = newPos;
            slider.setStyle("top", newPos.toString());
            
            // Stop the animation
            this._gadget.removeAttribute("running");
        }
        else
        {
            var speedY = this.speedY;
            
            var cy = this.currentYPos;
            var ty = this.targetYPos;
            
            // Calc new position
            var newPos = cy + speedY;
            
            // Factor decceleration    
            speedY *= .9;
            if ((speedY < 0) && (speedY > -.01))
            {
                // We've slowed to a stop - turn off animation
                this._gadget.removeAttribute("running");
                speedY = 0;
            }
            else if ((speedY >= 0) && (speedY < .01))
            {
                // We've slowed to a stop - turn off animation
                this._gadget.removeAttribute("running");
                speedY = 0;
            }
            
            // Limit and bounce appropriately at the end of travel
            if ((newPos + sliderHeight + this.overscan) < trackHeight)
            {
                newPos = (trackHeight - sliderHeight) - this.overscan;
                speedY = (speedY * .1);
            }
            else if (newPos > this.overscan)
            {
                speedY = -(speedY * .1);
                newPos = this.overscan;
            }
            
            // Move the box
            slider.setStyle("top", newPos.toString());
            
            // Update our vars
            this.lastYPos = this.currentYPos;
            this.currentYPos = newPos;
            var newSpeed = speedY;
            this.lastSpeedY = this.speedY;
            this.speedY = newSpeed;
        }        
        
        this.updateScrollButtons();
        
    }
}

//****************************************************************************

component.prototype.updateScrollButtons = function()
{
    var slider = this._gadget.getPartById("slider");
    var sliderHeight = slider.boxHeight;
    
    var track = this._gadget.getPartById("track");
    var trackHeight = track.boxHeight;

    var scrollUpButton = this._gadget.getPartById("scrollUpButton");
    var scrollDownButton = this._gadget.getPartById("scrollDownButton");

    if (sliderHeight <= trackHeight)
    {
        // If we aren't overflowed, we don't need to show either button
        scrollUpButton.disabled = true;
        scrollDownButton.disabled = true;
    }
    else
    {
        // Update the state of the up and down scroll buttons based on our position
        var top = slider.top;
        var bottom = top + sliderHeight;

        if (top > -1)
        {
            scrollUpButton.disabled = true;
            scrollDownButton.disabled = false;
        }
        else if ((bottom - 1) < trackHeight)
        {
            scrollUpButton.disabled = false;
            scrollDownButton.disabled = true;
        }
        else
        {
            scrollUpButton.disabled = false;
            scrollDownButton.disabled = false;
        }
    }
}

//****************************************************************************

component.prototype.onStartScroll = function(direction)
{
    if (!shell.currentEvent.targetBox.disabled)
    {
        var slider = this._gadget.getPartById("slider");
        var sliderHeight = slider.boxHeight;
        
        var track = this._gadget.getPartById("track");
        var trackHeight = track.boxHeight;

        if (sliderHeight > trackHeight)
        {
            // Stop the animation
            this._gadget.removeAttribute("running");
            
            if (this.showTransition && this.scene.prefObj.enableCarouselTransitions)
            {
                // Set up for our transition
                var snapshot1 = track.createSnapshot(false);
                snapshot1.addBoxRectItemsByTag(track, "convIcon");
                snapshot1.addBoxRectItemsByTag(track, "convAd");
            }
            var cy = this.currentYPos;
            
            // Calc new position
            var newPos = cy + (direction * this.iconHeight);
            
            // Limit and bounce appropriately at the end of travel
            if ((newPos + sliderHeight + this.overscan) < trackHeight)
            {
                newPos = (trackHeight - sliderHeight) - this.overscan;
            }
            else if (newPos > this.overscan)
            {
                newPos = this.overscan;
            }
            
            // Move the box
            slider.setStyle("top", newPos.toString());
            
            // Update our vars
            this.lastYPos = this.currentYPos;
            this.currentYPos = newPos;
            this.lastSpeedY = 0;
            this.speedY = 0;
            
            this.updateScrollButtons();
            
            if (this.showTransition && this.scene.prefObj.enableCarouselTransitions)
            {
                var snapshot2 = track.createSnapshot(true);
                track.attachTargetSnapshot(snapshot2);
                track.renderTargetSnapshot();
                snapshot2.addBoxRectItemsByTag(track, "convIcon");
                snapshot2.addBoxRectItemsByTag(track, "convAd");
                var transition = track.createTransition();
                if (transition)
                {
                    transition.type = "rectSorter";
                    transition.duration = 200;
                    transition.dynamicsType = "ease";
                    transition.fillColor = "#5d7074"
                    transition.perform(snapshot1, snapshot2);
                }       
                track.detachSourceSnapshot();
                track.detachTargetSnapshot();
                snapshot1 = snapshot2 = null;
            } 
        }
    }
}

//****************************************************************************

component.prototype.onEndScroll = function()
{
}

//****************************************************************************

component.prototype.addBuddy = function(buddyInfo, show)
{
    var newIcon;
    if (buddyInfo)
    {
        var buddyId = buddyInfo.aimId;
        var buddyDisplayId = buddyInfo.displayId;
        var buddyIcon = buddyInfo.buddyIcon;
        
        if (buddyId && (buddyId != "") && (buddyId != "undefined"))
        {
            buddyId = buddyId.toLowerCase();
        
            if (this._gadget.collapsed && show)
            {
                // Show the carousel if it's hidden        
                this.setVisible(true);
            }
            var icon = this.findBuddy(buddyId);
            if (icon) // make sure we only add each buddy once
            {
                // This icon is already in the list
                // If it's currently hidden, find the oldest icon besides this one and hide to so we have room to display THIS icon
                newIcon = icon;
                if (icon.collapsed)
                {
                    var snapshot1 = null, snapshot2 = null;

                    var oldestId = 99999;
                    var oldestIcon = null;
                    
                    var child = this._gadget.firstContentChild;
                    while (child)
                    {
                        if ((child != icon) && !(child.collapsed) && (child.getAttribute("age") < oldestId))
                        {
                            oldestId = child.getAttribute("age");
                            oldestIcon = child;
                        }
                        child = child.nextSibling;
                    }
                    
                    
                    // Hide the oldest icon
                    if (oldestIcon)
                    {
                        if (this.showTransition && this.scene.prefObj.enableCarouselTransitions)
                        {
                            // Set up for our transition
                            snapshot1 = this._gadget.createSnapshot(false);
                            snapshot1.addBoxRectItemsByTag(this._gadget, "convIcon");
                            snapshot1.addBoxRectItemsByTag(this._gadget, "convAd");
                        }
                        
                        // Hide the oldest icon
                        oldestIcon.collapsed = true;
                        
                        // Now show the newly-activated existing icon
                        icon.collapsed = false;
                        icon.setAttribute("age", this.age); this.age++; // update this icon's age since we just activated it
                        
                        if (this.showTransition && this.scene.prefObj.enableCarouselTransitions)
                        {
                            snapshot2 = this._gadget.createSnapshot(true);
                            this._gadget.attachTargetSnapshot(snapshot2);
                            this._gadget.renderTargetSnapshot();
                            snapshot2.addBoxRectItemsByTag(this._gadget, "convIcon");
                            snapshot2.addBoxRectItemsByTag(this._gadget, "convAd");
                            var transition = this._gadget.createTransition();
                            if (transition)
                            {
                                transition.type = "rectSorterZoom";
                                transition.duration = 500;
                                transition.dynamicsType = "spring";
                                transition.fillColor = "#5d7074"
                                transition.perform(snapshot1, snapshot2);
                            }       
                            this._gadget.detachSourceSnapshot();
                            this._gadget.detachTargetSnapshot();
                            snapshot1 = snapshot2 = null;
                        }
                    }
                }
            }
            else
            {
                var iconToHide = null;
                
                // Create a new icon
                newIcon = this.scene.createBox("convIcon");
                if (newIcon)
                {
                    // Fill in the icon's info
                    newIcon.setAttribute("buddyIcon", buddyIcon);
                    newIcon.setAttribute("buddyId", buddyId);
                    newIcon.setAttribute("buddyDisplayId", buddyDisplayId);
                    newIcon.setAttribute("label", buddyDisplayId);
                    newIcon.setAttribute("age", this.age); this.age++;
                    newIcon.setAttribute("unreadIMCount", "0");
                    newIcon.setAttribute("respectUIScale", this._gadget.getAttribute("respectUIScale"));
                    newIcon.setPresence(buddyInfo.state);

                    // If this is the first or only icon, activate it
                    if (this.numIcons == 0)
                        newIcon.setAttribute("active", "true");
                    
                    var snapshot1 = null, snapshot2 = null;
                    var doTransition = (this.scene.prefObj.enableCarouselTransitions && this.showTransition && this._gadget.contentChildCount >= 1 && show) ? true : false;
                    
                    if (doTransition)
                    {
                        // Set up for our transition
                        snapshot1 = this._gadget.createSnapshot(false);
                        snapshot1.addBoxRectItemsByTag(this._gadget, "convIcon");
                        snapshot1.addBoxRectItemsByTag(this._gadget, "convAd");
                    }                    

                    // Insert the new icon at the beginning of the list so newest conversations are always at the top                    
                    this._gadget.insertContentChildAt(newIcon, 0);
                    ++this.numIcons;
                    
                    // Hide the "oldest" icon if we need to make room
                    if (iconToHide)
                        iconToHide.collapsed = true;
                    
                    
                    // Do transition                
                    if (doTransition)
                    {
                        snapshot2 = this._gadget.createSnapshot(true);
                        this._gadget.attachTargetSnapshot(snapshot2);
                        this._gadget.renderTargetSnapshot();
                        snapshot2.addBoxRectItemsByTag(this._gadget, "convIcon");
                        snapshot2.addBoxRectItemsByTag(this._gadget, "convAd");
                        var transition = this._gadget.createTransition();
                        if (transition)
                        {
                            transition.type = "rectSorterZoom";
                            transition.duration = 600;
                            transition.dynamicsType = "spring4";
                            transition.fillColor = "#5d7074"
                            transition.perform(snapshot1, snapshot2);
                        }       
                        this._gadget.detachSourceSnapshot();
                        this._gadget.detachTargetSnapshot();
                        snapshot1 = snapshot2 = null;
                    }
                    // End transition stuff
                }
            }
        }
        
        this.onFrame(); // Update the bar position and scroll buttons
    }
    return newIcon;
}

//****************************************************************************

component.prototype.removeBuddy = function(buddyId)
{
    if (buddyId && (buddyId != "") && (buddyId != "undefined"))
    {
        buddyId = buddyId.toLowerCase();
    
        if (this.numIcons > 0)
        {
            var icon = this.findBuddy(buddyId);
            if (icon)
            {
                var newIcon;
                var navRight = false;
                // Find another icon to set active to once we've removed this one
                if (icon.getAttribute("active") == "true" && this._gadget.contentChildCount > 1)
                {
                    if (this._gadget.getAttribute("navRight").indexOf("buddyList") == -1)
                    {
                        // activate the previous icon if there is one
                        var presenceState;
                        var tempIcon = icon.previousSibling;
                        while(tempIcon)
                        {
                            presenceState = tempIcon.getAttribute("presenceState");
                            if (presenceState != "unknown" && presenceState != "1_update" && presenceState != "1_advertisement")
                            {
                                newIcon = tempIcon;
                                break;
                            }
                            else
                            {
                                tempIcon = tempIcon.previousSibling;
                            }
                        }
                        if (!newIcon)
                        {
                            // there wasn't a previous icon - activate the next icon if there is one
                            tempIcon = icon.nextSibling;
                            while(tempIcon)
                            {
                                presenceState = tempIcon.getAttribute("presenceState");
                                if (presenceState != "unknown" && presenceState != "1_update" && presenceState != "1_advertisement")
                                {
                                    newIcon = tempIcon;
                                    break;
                                }
                                else
                                {
                                    tempIcon = tempIcon.nextSibling;
                                }
                            }
                        }
                    }
                    else
                    {
                        this.navRightGadget();
                        navRight = true;
                    }
                    this.prevActiveBoxIdx = -1;
                }
                
                if(navRight && this._gadget.contentChildCount == 0)
                {
                    //if the user is in buddy list view and there is only a conversation and it's ad, then hide it
                    this.setVisible(false);   
                }
                else
                {
                    // if we are in conversation view
                    var realConvCount = this._gadget.contentChildCount - this.getFakeConversationCount();
                    if (newIcon && (realConvCount <= 2 && this.numIcons <= 2))
                    {
                        this.setVisible(false);   
                    }
                }
                
                if (newIcon)
                {
                    this.activateBuddy(newIcon.getAttribute("buddyId"));
                }
                
                var snapshot1=null, snapshot2=null;
                var doTransition = (this.scene.prefObj.enableCarouselTransitions && this.showTransition && !this._gadget.collapsed && this._gadget.contentChildCount > 1) ? true : false;
                
                if (doTransition)
                {
                    // Set up for our transition
                    snapshot1 = this._gadget.createSnapshot(false);
                    snapshot1.addBoxRectItemsByTag(this._gadget, "convIcon");
                    snapshot1.addBoxRectItemsByTag(this._gadget, "convAd");
                }

                // Delete the icon
                this._gadget.deleteContentChild(icon);
                --this.numIcons;
                
                // Do transition          
                if (doTransition)
                {
                    snapshot2 = this._gadget.createSnapshot(true);
                    this._gadget.attachTargetSnapshot(snapshot2);
                    this._gadget.renderTargetSnapshot();
                    snapshot2.addBoxRectItemsByTag(this._gadget, "convIcon");
                    snapshot2.addBoxRectItemsByTag(this._gadget, "convAd");
                    var transition = this._gadget.createTransition();
                    if (transition)
                    {
                        transition.type = "rectSorterZoom";
                        transition.duration = 800;
                        transition.dynamicsType = "spring4";
                        transition.fillColor = "#5d7074"
                        transition.perform(snapshot1, snapshot2);
                    }       
                    this._gadget.detachSourceSnapshot();
                    this._gadget.detachTargetSnapshot();
                    snapshot1 = snapshot2 = null;
                }
                // End transition stuff
                    
                if ((this.numIcons == 0) && (!this._gadget.collapsed))
                {
                    // collapse the entire carousel
                    this.setVisible(false);
                }
            }
        }
    }
    
    this.onFrame(); // Update the bar position and scroll buttons
    
}

//****************************************************************************

component.prototype.getFakeConversationCount = function()
{
    var count = 0;
    var child = this._gadget.firstContentChild;
    while (child)
    {
        presenceState = child.getAttribute("presenceState");
        if (child.hidden == false && (presenceState == "unknown" || presenceState == "1_update"))
        {
            count++;    
        }
        child = child.nextSibling;
    }
    return count;
}

//****************************************************************************

component.prototype.findBuddy = function(buddyId)
{
    if (buddyId && (buddyId != "") && (buddyId != "undefined"))
    {
        buddyId = buddyId.toLowerCase();
    
        var child = this._gadget.firstContentChild;
        while (child)
        {
            if (child.getAttribute("buddyId") == buddyId)
            {
                return child;
            }
            child = child.nextSibling;
        }
    }
    
    return null;
}

//****************************************************************************

component.prototype.removeActiveBuddy = function()
{
    this.removeBuddy(this.getActiveBuddy());
    this.onFrame(); // Update the bar position and scroll buttons
}

//****************************************************************************

component.prototype.removeAllBuddies = function()
{
    // collapse the entire carousel since we won't have any more icons
    //this._gadget.collapsed = true;
    this.setVisible(false);
    
    // delete the icons
    this._gadget.deleteContentChildren();
    this.onFrame(); // Update the bar position and scroll buttons
}

//****************************************************************************

component.prototype.activateBuddy = function(buddyId, activateConversation)
{
    var actived = null;
    var i = 0;
    
    if (buddyId && (buddyId != "") && (buddyId != "undefined"))
    {    
        buddyId = buddyId.toLowerCase();
        
        var child = this._gadget.firstContentChild;
        var currActived = -1;
        
        //TODO use variable to remember the last active box instead while loop here
        while (child)
        {
            if (child.getAttribute("buddyId") == buddyId)
            {
                currActived = i;
                child.setAttribute("active", "true");
                child.setAttribute("age", this.age); this.age++; // update the icon's age since we just activated this icon
                child.setAttribute("unreadIMCount", "0");
                child.setIMCount(0);
                actived = child;
            }
            else
            {
                child.setAttribute("active", "false");
            }
            
            child = child.nextSibling;
            ++i;
        }
    }
    
    var presenceState = actived ? actived.getAttribute("presenceState") : "";    
    if (this._gadget.contentChildCount == 1 && presenceState != "1_advertisement")
    {
        this.setVisible(false);   
    }
    if (activateConversation == undefined)
    {
        activateConversation = true;
    }
    if (activateConversation && actived && currActived != -1)
    {    
        var convWindow = this.scene.getBoxById("chatPanel");
        var event = shell.factory.createEvent(actived);
        if (event)
        {	    
            // Send an event to the conversation window that we active a buddy
            event.prevActiveIdx = this.prevActiveBoxIdx;
            event.currActiveIdx = currActived;
            this.prevActiveBoxIdx = currActived;
            
            if (presenceState == "unknown")
            {
                event.type = "unknownConversation";
                event.screenName = actived.getAttribute("buddyId");
            }
            else if (presenceState == "1_update")
            {
                event.type = "softwareUpdateIconActivate";
            }
            else if (presenceState == "1_advertisement")
            {
                event.type = "advertisementActivated";
            }
            else
            {
                event.type = "buddyTabActivated";
                event.screenName = actived.getAttribute("buddyId");
            }
            convWindow.dispatchEvent(event);
        }
    }
    
    return actived;
}

//****************************************************************************

component.prototype.getActiveBuddyBox = function()
{
    var child = this._gadget.firstContentChild;
    while (child)
    {
        if (child.getAttribute("active") == "true")
        {
            return child;
        }
        child = child.nextSibling;
    }
    
    return null;
}

//****************************************************************************

component.prototype.getFocusedBuddyBox = function()
{
    var child = this._gadget.firstContentChild;
    while (child)
    {
        if (child.getAttribute("focused") == "true")
        {
            return child;
        }
        child = child.nextSibling;
    }
    
    return null;
}

//****************************************************************************

component.prototype.getNumIcons = function()
{
    return this._gadget.contentChildCount;
}

//****************************************************************************

component.prototype.getActiveBuddy = function()
{
    var buddy = this.getActiveBuddyBox();
    return buddy ? buddy.getAttribute("buddyId") : "";
}

//****************************************************************************

component.prototype.setVisible = function(visible)
{
    if (this._gadget.collapsed == visible)
    {
        var doTransition = this.showTransition && this.scene.prefObj.enableAllTransitions;
        if (doTransition)
        {
            var snapshot1 = null;
            var snapshot2 = null;
                
            var box = this.scene.getBoxById("containerContainer");
            snapshot1 = box.createSnapshot(false); // grab what's on the screen as our 1st snapshot
            box.attachSourceSnapshot(snapshot1);       // This box will now draw on-screen from the snapshot
            snapshot2 = box.createSnapshot(true); // create a blank snapshot object
            box.attachTargetSnapshot(snapshot2);      // now we are going to be drawing INTO the 2nd snapshot
        }
    
        this._gadget.collapsed = visible ? false : true;
        
        if (doTransition)
        {
            box.renderTargetSnapshot();
            if (snapshot1 && snapshot2)
            {
                // Create a transition
                var transition = box.createTransition();
                
                if (transition)
                {
                    // Do the transition
                    transition.type = "dissolve";
                    transition.duration = 400;
                    transition.dynamicsType = "ease";
                    transition.perform(snapshot1, snapshot2);
                }       
                box.detachSourceSnapshot();
                box.detachTargetSnapshot();
                snapshot1 = snapshot2 = null;
            } 
        }
    }
}

//****************************************************************************

component.prototype.onReceivedIM = function(buddyId)
{
    // Increment the unread IM count
    var icon = this.findBuddy(buddyId);
    if (icon)
    {
        var count = parseInt(icon.getAttribute("unreadIMCount")) + 1;
        icon.setIMCount(count);
        icon.setAttribute("unreadIMCount", "" + count);
        
        this.moveBuddyToTop(buddyId, icon);
    }
}

//****************************************************************************

component.prototype.moveBuddyToTop = function(buddyId, iconBox)
{
    var icon = iconBox;
    if (!icon)
        icon = this.findBuddy(buddyId);

    if (icon)
    {
        var index = this.getIconIndex(buddyId);
        if (index > 0)
        {
            // Set up for our transition
            if (this.showTransition && this.scene.prefObj.enableCarouselTransitions)
            {
                var snapshot1 = this._gadget.createSnapshot(false);
                snapshot1.addBoxRectItemsByTag(this._gadget, "convIcon");
                snapshot1.addBoxRectItemsByTag(this._gadget, "convAd");
            }

            // Reposition this icon to the top of the carousel
            this._gadget.removeContentChild(icon);
            this._gadget.insertContentChildAt(icon, 0);

            //
            // Workaround for boxely bug -     
            //
            // When we do a removeContentChild and then insertContentChildAt (see above)
            // on the icon to reposition it in the carousel, boxely seems to be losing any
            // parts that have position="fixed". Here we're recreating the badge part (which
            // needs to be fixed position) and adding the a new badge to the icon after we've
            // repositioned it.
            // 
            // We can mitigate the bug by adding layer="stack" to the fixed position part,
            // but then it doesn't work correctly with our transition. :-(
            //
            {
                var oldBadge = icon.getBoxByTag("convIconBadge");
                var newBadge = this.scene.createBox("convIconBadge");
                
                newBadge.setAttribute("label", oldBadge.getAttribute("label"));
                newBadge.setAttribute("image", oldBadge.getAttribute("image"));
                newBadge.setAttribute("respectUIScale", oldBadge.getAttribute("respectUIScale"));
                newBadge.setAttribute("showBadge", "true");
                newBadge.setAttribute("showLabel", "true");
                
                // Swap the badges
                icon.deleteChild(oldBadge);
                icon.insertChildAt(newBadge, -1);
            }            
           
            // Scroll the the top of the list (which is where the icon is now)
            var slider = this._gadget.getPartById("slider");
            slider.setStyle("top", "0");
            
            if (this.showTransition && this.scene.prefObj.enableCarouselTransitions)
            {
                // Do the transition            
                var snapshot2 = this._gadget.createSnapshot(true);
                this._gadget.attachTargetSnapshot(snapshot2);
                this._gadget.renderTargetSnapshot();
                snapshot2.addBoxRectItemsByTag(this._gadget, "convIcon");
                snapshot2.addBoxRectItemsByTag(this._gadget, "convAd");
                
                var transition = this._gadget.createTransition();
                if (transition)
                {
                    transition.type = "rectSorter";
                    transition.duration = 500;
                    transition.dynamicsType = "ease";
                    transition.fillColor = "#5d7074"
                    transition.perform(snapshot1, snapshot2);
                }       
                this._gadget.detachSourceSnapshot();
                this._gadget.detachTargetSnapshot();
                snapshot1 = snapshot2 = null;
            }
            // Update the scroll buttons
            this.updateScrollButtons();
        }
    }
}

//****************************************************************************

component.prototype.updateBuddyPresence = function(buddyInfo)
{
    var buddyId = buddyInfo.aimId;
    var state = buddyInfo.state;
    var icon = this.findBuddy(buddyId);
    if (icon)
    {
        icon.setAttribute("label", buddyInfo.displayId);
        icon.setPresence(state);
        if (buddyInfo.buddyIcon)
        {
            icon.setAttribute("buddyIcon", buddyInfo.buddyIcon);
        }
    }
    return icon;
}

//****************************************************************************

component.prototype.activateSelectedBuddy = function()
{
    var selectBox = this._gadget.getPartById("slider");
    var index = selectBox.cursorIndex;
    var box = this.getBuddyBoxByIndex(index);
    if (box)
    {
        var buddyId = box.getAttribute("buddyId");
        if (buddyId && buddyId != "")
        {
            // Make the icon look pressed
            var icon = this.findBuddy(buddyId);
            if (icon)
                icon.setAttribute("pressed", "true");
                
            // Activate it
            this.activateBuddy(buddyId);
            
            // Unpress the icon
            if (icon)
                icon.setAttribute("pressed", "false");
        }
    }
}

//****************************************************************************

component.prototype.getBuddyBoxByIndex = function(index)
{
    var count = 0;
    
    var child = this._gadget.firstContentChild;
    while (child)
    {
        if (count == index)
        {
            return child;
        }
        ++count;
        child = child.nextSibling;
    }
    
    return null;
}

//****************************************************************************

component.prototype.getIconIndex = function(buddyId)
{
    var count = 0;
    
    var child = this._gadget.firstContentChild;
    while (child)
    {
        var id = child.getAttribute("buddyId");
        if (buddyId == id)
        {
            return count;
        }
        ++count;
        child = child.nextSibling;
    }
    
    return -1;
}

//****************************************************************************

component.prototype.navRightGadget = function()
{
    var e = shell.currentEvent;
    var navStr = this._gadget.getAttribute("navRight");

    var navBox = navStr;
    var navPart = "";
    // Look for gadget part reference
    if (navStr && navStr.search("[.]"))
    {
        var navSplit = navStr.split(".");
        navBox  = navSplit[0];
        navPart = navSplit[1];
    }
    if (e)
    {
        e.stopPropagation();
    }

    if (navBox == "chatPanel")
    {
        var convWindow = this.scene.getBoxById("chatPanel");
        var currConv = convWindow.getCurrConversation();
        if (currConv)
        {
            currConv.setComposeInputFocus();
        }
    }
    else
    {
        var nextBoxRef = this.scene.getBoxById(navBox);
        if (nextBoxRef && nextBoxRef.collapsed == false && nextBoxRef.hidden == false)
        {
            if (navPart)
            {
                nextBoxRef = nextBoxRef.getPartById(navPart);
            }
            nextBoxRef.focus();
            this.scene.focusedBox = nextBoxRef;
        }
    }
}

//****************************************************************************

component.prototype.onFocused = function()
{
    var selectBox = this._gadget.getPartById("slider");
    var focusedBox = this.scene.focusedBox;
    if (focusedBox && (focusedBox != selectBox))
    {
        this.scene.focusedBox = selectBox;
        
        var child = this._gadget.firstContentChild;
        if (selectBox.selectedBox != child)
        {
            selectBox.selectedBox = child;
        }
    }
}

//****************************************************************************

component.prototype.trackBlur = function()
{
    var selectBox = this._gadget.getPartById("slider");
    var child = this._gadget.firstContentChild;
    if (child)
    {
        selectBox.selectedBox = child;
        this.ensureIconVisible(child.getAttribute("buddyId"));
    }
}

//****************************************************************************

component.prototype.focusChanged = function()
{
    var focusedBox = this.getFocusedBuddyBox();
    if (focusedBox)
    {
        // Bring the focused icon into focus
        var buddyId = focusedBox.getAttribute("buddyId");
        this.ensureIconVisible(buddyId);
    }
}

//****************************************************************************

component.prototype.ensureIconVisible = function(buddyId)
{
    var slider = this._gadget.getPartById("slider");
    var sliderHeight = slider.boxHeight;
    
    var track = this._gadget.getPartById("track");
    var trackHeight = track.boxHeight;

    if (sliderHeight > trackHeight)
    {
        var index = this.getIconIndex(buddyId);
        if (index != -1)
        {
            var iconHeight = this.iconHeight;
            
            var top = slider.top;
            var bottom = top + sliderHeight;

            var iconTop = top + (index * iconHeight);
            
            // The ad is a different height than the buddy icons            
            // We can assume the ad is the last item in the carousel
            if (buddyId == "1_advertisement")
            {
                var ad = this._gadget.getBoxByTag("convAd");
                iconTop = iconTop + (ad.boxHeight - iconHeight);
                iconHeight = ad.boxHeight;
            }

            var iconBottom = iconTop + iconHeight;
            
            if ((iconTop < 0) || (iconBottom > trackHeight))
            {
                var doTransition = this.scene.prefObj.enableCarouselTransitions && this.showTransition && !this._gadget.collapsed;
                if (doTransition)
                {
                    // Set up for our transition
                    var snapshot1 = this._gadget.createSnapshot(false);
                    snapshot1.addBoxRectItemsByTag(this._gadget, "convIcon");
                    snapshot1.addBoxRectItemsByTag(this._gadget, "convAd");
                }
                
                if (iconTop < 0)
                {
                    // Off the top - scroll the list up
                    var cy = -(index * iconHeight);
                    slider.setStyle("top", cy.toString());
                }
                else if (iconBottom > trackHeight)
                {
                    // Off the bottom - scroll the list down
                    var bottomEdge = top + trackHeight;
                    var cy = -(iconBottom - bottomEdge);
                    slider.setStyle("top", cy.toString());
                }
                // else in view already
                
                if (doTransition)
                {
                    var snapshot2 = this._gadget.createSnapshot(true);
                    this._gadget.attachTargetSnapshot(snapshot2);
                    this._gadget.renderTargetSnapshot();
                    snapshot2.addBoxRectItemsByTag(this._gadget, "convIcon");
                    snapshot2.addBoxRectItemsByTag(this._gadget, "convAd");
                    var transition = this._gadget.createTransition();
                    if (transition)
                    {
                        transition.type = "rectSorter";
                        transition.duration = 200;
                        transition.dynamicsType = "ease";
                        transition.fillColor = "#5d7074"
                        transition.perform(snapshot1, snapshot2);
                    }       
                    this._gadget.detachSourceSnapshot();
                    this._gadget.detachTargetSnapshot();
                    snapshot1 = snapshot2 = null;
                }
                this.updateScrollButtons();
            }            
        }
        // else buddyId not found
    }
}

//****************************************************************************

component.prototype.deactivateAllIcons = function()
{
    var child = this._gadget.firstContentChild;
    while (child)
    {
        child.setAttribute("active", "false");
        child = child.nextSibling;
    }
}

//****************************************************************************



