// ****************************************************************************
//
// Copyright ?2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************

function component(_gadget)
{
    this.lastSelectedMessageIdx = -1;
    this.transferingFile = false;
    
    //we need this because when conversation is deleted while the user is editing, it call focusLost
    //which opens another can of warms
    this.deleting = false;
}

component.prototype.closeConversation = function()
{
    if (this._gadget.gadgetParent)
    {
        this.deleting = true;
        this._gadget.gadgetParent.closeConversationHelper(this._gadget);
    }
}

component.prototype.setComposeInputFocus = function()
{
    var focusedBox = this.scene.focusedBox;
    if (!focusedBox || focusedBox.tag != "composeInput")
    {
        var smileyPalette = this.scene.getBoxById("smileyPalette");
        if (!smileyPalette.collapsed)
            smileyPalette.toggle();

        var input = this._gadget.getPartById("composeInput");
        this.scene.focusedBox = input;
        var inputValue = input.getAttribute("value");
        if (inputValue)
        {
            input.setCaretPos(inputValue.length);
        }
    }
}

component.prototype.sendIM = function()
{
    if (this._gadget.getAttribute("disableChat") == "false")
    {
        // Get the compose text
        var composeInput = this._gadget.getPartById("composeInput");
        var message = composeInput.getAttribute("value");
        
        if (message)
        {
            var isReconnecting = (this.scene.AIMobj.getIsReconnect() || this.scene.AIMobj.getContinueOffline())
            var event = shell.factory.createEvent(this._gadget);
            if (event)
            {	 
                event.type = "sendIM";
                event.reconnecting = (isReconnecting) ? true : false;
                if (!isReconnecting)
                {
                    composeInput.setAttribute("value", "");
                    event.screenName = this._gadget.id;
                    event.message = message;
                }
                this._gadget.dispatchEvent(event);
            }

            if (!isReconnecting)
            {
                // Clear out the compose text
                composeInput.setStyle("height", "17*");
                composeInput.setStyle("maxHeight", "17*");
            
                var signedInName = this._gadget.getAttribute("signedInScreenName");
                signedInName = (signedInName && signedInName!=undefined) ? signedInName : "";
                
                // Also add it to the history window
                var conversation = this._gadget.getPartById("conversationHistory");
                conversation.addSentIM(signedInName, message);

                try {
                    var kvs = new keyValueStorage();
                    var store = this.scene.prefObj.prefsGetStoreForCurrentIdentity();
                    if (store)
                    {
                        kvs.open( store );
                        var playSendIMSound = kvs.getValue(this.scene.prefObj.playSendIMSoundKey, this.scene.prefObj.playSendIMSoundDefaultValue);
                        kvs.close();
                        if (playSendIMSound.toString() == "true")
                        {
                            var path = shell.app.path + "\\resources\\imsend.wav";
                            shell.device.playSoundOrVibrate(path, 0, 0, 0); // no need to vibrate when we send a msg
                        }
                    }
                }
                catch (e){}
            }
        }
    }
}

component.prototype.onSmileyPanel = function()
{
    if (this._gadget.getAttribute("disableChat") == "false")
    {
        var palette = this.scene.getBoxById("smileyPalette");
        palette.toggle();
    }
}

component.prototype.insertComposeText = function(string)
{
    var input = this._gadget.getPartById("composeInput");
    input.replaceSelection(string);
    this.scene.focusedBox = input;
    var value = input.getAttribute("value");
    if (value)
    {
        input.setCaretPos(value.length);
    }
}

component.prototype.getDisplayId = function()
{
    return this._gadget.getPartById("screenName").getAttribute("value");
}

component.prototype.getBuddyInfo = function()
{
    var buddyInfo = new Object();
    buddyInfo.aimId = this._gadget.id;
    buddyInfo.displayId = this._gadget.getPartById("screenName").getAttribute("value");
    buddyInfo.buddyIcon = this._gadget.getAttribute("buddyIcon");
    buddyInfo.statusMsg = this._gadget.getPartById("statusMessage").getAttribute("value");
    buddyInfo.state = this._gadget.getAttribute("presenceState");
    buddyInfo.presenceIcon = this._gadget.getAttribute("presenceIcon");
    
    return buddyInfo;
}

component.prototype.setBuddyInfo = function(buddyInfo, manualUpdate)
{
    this._gadget.id = buddyInfo.aimId;
    this._gadget.getPartById("screenName").setAttribute("value", buddyInfo.displayId);
    
    if (buddyInfo.buddyIcon)
    {
        this._gadget.setAttribute("buddyIcon", buddyInfo.buddyIcon);
    }
    
    var statusMessage = (buddyInfo.state == "mobile" && !buddyInfo.statusMsg) ? this.scene.rootBox.getAttribute("im.status.mobile") : buddyInfo.statusMsg;
    if (statusMessage != this._gadget.getPartById("statusMessage").getAttribute("value"))
    {
        this._gadget.getPartById("statusMessage").setAttribute("value", statusMessage);
    }
    if (buddyInfo.state) 
    {
        this.updateStatus(buddyInfo.state, manualUpdate);
    
        //Set the presenceIcon
        this._gadget.setAttribute("presenceIcon", getPresenceIcon(buddyInfo.state));
    }
}

component.prototype.onModified = function()
{   
    var value = this._gadget.getPartById("composeInput").getAttribute("value");
    if (value == undefined || value == "" || value.length == 1)
    {
        var event = shell.factory.createEvent(this._gadget);
        if (event)
        {	    
            event.type = "inputModified";
            this._gadget.dispatchEvent(event);
        }
    }
    
    var inputBox = this._gadget.getPartById("composeInput");
    var lineCount = inputBox.lineCount;
    var newHeight = (lineCount < 3) ? 17 + (13 * (lineCount -1)) : 43;
    
    if (inputBox.height != newHeight)
    {
        inputBox.setStyle("height", newHeight + "*");
        inputBox.setStyle("maxHeight", newHeight + "*");
    }
}

component.prototype.setEditing = function(editing)
{
    if (!this.deleting)
    {
        this._gadget.setAttribute("editing", editing ? "true" : "false");
        var screenName = this._gadget.getPartById("screenName");
        screenName.collapsed= editing;
        var input = this._gadget.getPartById("newMessageInput");
        input.collapsed = !screenName.collapsed;
        if (editing)
        {
            this.scene.focusedBox = input;
            if (this._gadget.getAttribute("mobileConversation") == "true")
            {
                var areaCode = "+1 ";
                input.setAttribute("value", areaCode);
                input.inputMode = 7;
                input.setCaretPos(areaCode.length);
            }
        }
        else
        {
            var convWindow = this.scene.getBoxById("chatPanel");
            propagateEventForBox(convWindow, "editComplete"); // update the menu
        }
    }
}

component.prototype.onInputFocusLost = function()
{
    var input = this._gadget.getPartById("newMessageInput");
    if (!input.focused)
    {
        this.setNewMessage();
    }
}

component.prototype.setNewMessage = function()
{
    if (!this.deleting && this._gadget.getAttribute("editing") == "true")
    {
        this._gadget.setEditing(false);
        var input = this._gadget.getPartById("newMessageInput");
        var value = input.getAttribute("value");
        if (value)
        {
            value = value.toLowerCase();
            value = value.replace(/[- \t]*/ig,"");
        }
        if (!value || value == "")
        {
            this._gadget.closeConversation();
        }
        else
        {
            if (this._gadget.getAttribute("mobileConversation") == "true")
            {   
                if (value.indexOf("+1") != 0)
                {
                    var firstChar = value.substr(0, 1);
                    if (firstChar == "1")
                    {
                        value = "+" + value;
                    }
                    else if (firstChar == "+")
                    {
                        if (value.substr(1, 1) != "1")
                        {
                            value = value.replace("+", "+1");
                        }
                    }
                    else
                    {
                        value = "+1" + value;
                    }
                }
            }
            this._gadget.setAttribute("disableChat", "false");
            this._gadget.getPartById("screenName").setAttribute("value", value);
            propagateEventForBox(this._gadget, "newMessageScreenNameSet");
            var focusedBox = this.scene.focusedBox;
            
            if (focusedBox && focusedBox.id == "newMessageInput")
            {
                this.setComposeInputFocus();
            }
        }
    }
}   

component.prototype.updateStatus = function(status, manualUpdate)
{
    var currentPresence = this._gadget.getAttribute("presenceState");
    if (currentPresence != status)
    {
        var message;
        var rootBox = this.scene.rootBox;
        var conversation = this._gadget.getPartById("conversationHistory");
        var screenName = this._gadget.getPartById("screenName").getAttribute("value");
        if (currentPresence)
        {
            if (currentPresence == "blocked" || currentPresence == "unblocked")
            {
                manualUpdate = false;
            }
        }
        
        if (!manualUpdate)
        {
            var generateMessage = true;
            switch(status)
            {
                case "online":
                {
                    if (this._gadget.getAttribute("presenceState") != "offline")
                    {
                        message = "im.status.returnOnline";       
                    }
                    else
                    {
                        message = "im.status.online";
                    }
                    break;
                }
                case "away":
                {
                    message = "im.status.away";
                    break;
                }
                case "idle":
                {
                    message = "im.status.idled";
                    break;
                }
                case "offline":
                {
                    message = formatString2(rootBox.getAttribute("im.status.offline"), screenName, conversation.getTimeString());
                    conversation.addAlertIM(message);
                    generateMessage = false;
                    break;
                }
                case "blocked":
                {
                    message = "im.status.offline";
                    break;
                }
            }
            if (generateMessage && message)
            {
                message = formatString2(rootBox.getAttribute(message), screenName, conversation.getTimeString());
            }
        }
        
        if (status == "offline")
        {
            message = formatString1(rootBox.getAttribute("im.status.offlineMessage"), screenName);
        }
        if (message)
        {
            conversation.addAlertIM(message);
        }
        this._gadget.setAttribute("presenceState", status);
    }
}

component.prototype.addAlertToConversation = function(message)
{
    var conversation = this._gadget.getPartById("conversationHistory");
    conversation.addAlertIM(message);
}

component.prototype.setConversationHistoryFocus = function(isUp)
{
    var conversationHistory = this._gadget.getPartById("conversationHistory");
    var focusedBox = "conversationHistory";
    if (conversationHistory.contentChildCount == 0)
    {
        focusedBox = isUp ? "screenName" : "composeInput";
    }
    else
    {
        conversationHistory.selectedBox = isUp ? conversationHistory.lastContentChild : conversationHistory.firstChild;
    }
    this.setGadgetFocus(focusedBox);
    this.lastSelectedMessageIdx = (conversationHistory.contentChildCount != 0 && isUp) ? conversationHistory.contentChildCount - 1 : 0;
}

component.prototype.setGadgetFocus = function(part)
{
    this.scene.focusedBox = this._gadget.getPartById(part);
}

component.prototype.setSmileyPaletteFocus = function()
{
    var smileyPalette = this.scene.getBoxById("smileyPalette");
    if (smileyPalette.collapsed)
        smileyPalette.toggle();

    if (shell.currentEvent)
    {
        shell.currentEvent.preventDefault();
    }        
    this.scene.focusedBox = smileyPalette.getPartById("button1");
}

component.prototype.onSmileyButtonDown = function()
{
    var smileyPalette = this.scene.getBoxById("smileyPalette");
    if (!smileyPalette.collapsed)
    {
        smileyPalette.toggle();
    }
    else
    {
        this.setGadgetFocus("screenName");
    }
}

component.prototype.navUpGadget = function()
{
    var currSelectedIndex = this._gadget.getPartById("conversationHistory").selectedIndex;
    if (currSelectedIndex == 0 && this.lastSelectedMessageIdx == currSelectedIndex)
    {
        this.setGadgetFocus("screenName");
    }
    else
    {
        this.lastSelectedMessageIdx = currSelectedIndex - 1
    }
}

component.prototype.navDownGadget = function()
{
    var conversationHistory = this._gadget.getPartById("conversationHistory");
    var currSelectedIndex = conversationHistory.selectedIndex;
    var currCount = conversationHistory.contentChildCount;
    var cursorIndex = conversationHistory.cursorIndex;
    
    if (currSelectedIndex == currCount - 1)
    {
        this.setComposeInputFocus();
    }
    else
    {
        this.lastSelectedMessageIdx = currSelectedIndex;
    }
}

component.prototype.sendPicture = function()
{
    if (!this.transferingFile)
    {
        var photoSpec;
        try
        {
            photoSpec = shell.device.browseForImage("Thumbnail View", true);
        }
        catch(e){}
        if (photoSpec)
        {
            var alertBox = this.scene.getBoxById("alertBox");
            alertBox.alert(null, "alertContentSendPicture", "Send", "Cancel", this, "sendPictureDialogSend");
            var image = alertBox.getBoxByTag("alertContentSendPicture");
            image.setAttribute("src", photoSpec);
            //shell.device.viewImage(photoSpec);
        }
    }
}

component.prototype.sendPictureDialogSend = function(buttonIndex)
{
    if (buttonIndex == 0)
    {
        // Pull out the settings from the alert
        var alertBox = this.scene.getBoxById("alertBox");
        
        var imagePath = alertBox.getBoxByTag("alertContentSendPicture").getAttribute("src");
        if (imagePath)
        {
            var conversation = this._gadget.getPartById("conversationHistory");
            conversation.addAlertIM("Waiting for picture to be accepted.");
            this.transferingFile = true;
        }
    }
    this.setComposeInputFocus();
}

component.prototype.leftPressed = function()
{
    propagateEventForBox(this._gadget, "navLeft");
}

component.prototype.onKeyDownOutsideComposeInput = function()
{
    var keyCode = shell.currentEvent.keyCode;
    if ((keyCode >= 32 && keyCode <= 126))
    {
        this.setComposeInputFocus();
    }
}

component.prototype.onKeyDown = function()
{
    var virtualKey = shell.currentEvent.virtualKeyCode;

    // left = 37, up = 38, right = 39, down = 40
    var vkLeft = 37, vkUp = 38, vkRight=39, vkDown = 40;
    if ((virtualKey >= vkLeft) && ( virtualKey <= vkDown))
    {
        // get the current caret position
        var inbox = this._gadget.getPartById("composeInput");
        if(inbox)
        {
            try
            {
                var caretPos = inbox.getCaretPos();
                var caretY = inbox.getLineFromPos(-1);
                if((caretY == 0) && (virtualKey == vkUp))
                {
                    this.setConversationHistoryFocus(true);
                    shell.currentEvent.stopPropagation();
                }
                else if((caretPos == 0) && (virtualKey == vkLeft))
                {
                    this.leftPressed();
                }
                else if((caretPos == inbox.value.length) && (virtualKey == vkRight))
                {
                    this.setGadgetFocus("smileyButton");
                }
                else if ((caretY == inbox.lineCount -1) && (virtualKey == vkDown))
                {
                    this.setGadgetFocus("screenName");
                    shell.currentEvent.stopPropagation();
                }
            }
            catch(e){}
        }
    }
}

component.prototype.menuItemSelected = function()
{
    var event = shell.currentEvent;
    if (event)
    {
        var gadgetParent = this._gadget.gadgetParent;
        
        switch(event.context)
        {
            case "close":
            {
                this.closeConversation();
                break;
            }
            case "info":
            {
                this.scene.focusedBox = this._gadget.getPartById("composeInput");
                gadgetParent.getProfileForCurrConversation();
                break;
            }
            case "block":
            {
                gadgetParent.blockUnblockBuddyById(this._gadget.id, true);
                break;
            }   
            case "add":
            {
                gadgetParent.addCurrentConversation();
                break;
            }
            case "report":
            {
                gadgetParent.reportSpamDialogOnCurrentUser();
                break;
            }
        }
    }
}

component.prototype.toggleDropDownLabel = function()
{
    var targetBox = shell.currentEvent.targetBox;
    var globalDropDown = this.scene.getBoxById("globalDropDown");
    if (globalDropDown.collapsed)
    {
        var rootBox = this.scene.rootBox;
        var blockText = this._gadget.getAttribute("presenceState") == "blocked" ? "im.menu.label.Unblock" : "im.menu.label.Block";
        blockText = rootBox.getAttribute(blockText);
        if (globalDropDown.getParentBox() != targetBox)
        {
            globalDropDown.setParentBox(targetBox);
            globalDropDown.appendMenuItem(rootBox.getAttribute("im.menu.label.closeConversation"), null, "close");
            globalDropDown.appendMenuItem(rootBox.getAttribute("bl.menu.label.BuddyInfo"), null, "info");
            if (!this.scene.AIMobj.getContinueOffline())
            {
                globalDropDown.appendMenuItem(blockText, null, "block");
                globalDropDown.appendMenuItem(rootBox.getAttribute("bl.addBuddy"), null, "add");
                globalDropDown.appendMenuItem(rootBox.getAttribute("im.menu.label.ReportSpam"), null, "report");
            }
        }
        else
        {
            var blockMenuItem = globalDropDown.getMenuItemByIndex(2);
            if (blockMenuItem)
            {
                blockMenuItem.setAttribute("value", blockText);
            }
        }
        globalDropDown.layoutDropdown();
        globalDropDown.setSelectedBoxByIndex(0, false);
    }
    
    targetBox.setAttribute("selected", globalDropDown.collapsed ? "true" : "false");
    globalDropDown.collapsed = !globalDropDown.collapsed;
}

component.prototype.screenNameLayout = function()
{
    var scale = shell.uiScale / 100;
    var topHat = this._gadget.getPartById("topHat");
    var dropDownLabel = this._gadget.getPartById("screenName");
    var screenNameLabel = dropDownLabel.getPartById("label");
    var dropdownIcon = dropDownLabel.getPartById("presenceButton");
    
    //work around for maxWidth="auto" not working.  temporary fix.
    screenNameLabel.maxWidth = topHat.boxWidth - (66 * scale);
    dropDownLabel.maxWidth = screenNameLabel.boxWidth + (16 * scale);
    
    var globalDropDown = this.scene.getBoxById("globalDropDown");
    if (!globalDropDown.collapsed)
    {
        var parentBox = globalDropDown.getParentBox();
        if (parentBox && parentBox.gadgetParent && parentBox.gadgetParent.tag == "conversation")
        {
            globalDropDown.layoutDropdown();
        }
    }
    
}

component.prototype.onDropdownClosed = function()
{
    var focusedBox = this.scene.focusedBox;
    if (focusedBox && focusedBox.gadgetParent && focusedBox.gadgetParent.id == "dropDown")
    {
        this.scene.focusedBox = this._gadget.getPartById("screenName");
    }
}

