// ****************************************************************************
//
// Copyright ?2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************

function component (_gadget)
{
}

component.prototype.addReceivedIM = function(buddyId, message, isAutoResponse)
{
    message = this.parseMessage(message);
    this.addMessageMessage(buddyId, message, "remote", isAutoResponse);
}

component.prototype.addReceivedOfflineIM = function(buddyId, message, timestamp)
{
    message = this.parseMessage(message);
    this.addOfflineMessage(buddyId, message, "remote", timestamp);
}

component.prototype.addSentIM = function(buddyId, message)
{
    message = this.parseMessage(message);
    this.addMessageMessage(buddyId, message, "local");
}

component.prototype.addReceivedPicture = function(buddyId, src)
{
    this.addMessagePicture(buddyId, src, "remote");
}

component.prototype.addSentPicture = function(buddyId, src)
{
    this.addMessagePicture(buddyId, src, "local");
}

component.prototype.addAlertIM = function(message)
{
    var item = this.scene.createBox("conversationHistoryAlertItem");
    if (item)
    {
        item.setAttribute("newMessage", message);
        
        // Add the new message block and scroll to it
        this._gadget.addContentChild(item);
        this.scrollToNewItem(item);
    }
}

component.prototype.addMessagePicture = function(buddyId, src, identity)
{
    // Create a new message block
    var item = this.scene.createBox("conversationHistoryImageItem");
    if (item)
    {
        // Add the properties to this message block
        item.setAttribute("buddyId", buddyId);
        item.setAttribute("time", this.getTimeString());
        item.setAttribute("src", src);
        item.setAttribute("identity", identity);
        
        // Add the new message block and scroll to it
        this._gadget.addContentChild(item);
        this.scrollToNewItem(item);
    }
}

component.prototype.addMessageMessage = function(buddyId, message, identity, isAutoResponse)
{
    var timestamp = this.getTimestamp();
    
    var lastMessage = this.getLastMessage();
    if (lastMessage)
    {
        var lastBuddyId = lastMessage.getAttribute("buddyId");
        var lastIdentity = lastMessage.getAttribute("identity");
        if ((lastBuddyId == buddyId) && (lastIdentity == identity))
        {
            var lastTimestamp = parseInt(lastMessage.getAttribute("timestamp"));
            var elapsedMins = (timestamp - lastTimestamp) / 60;
            if (elapsedMins < 1)
            {
                // Append to the last message
                lastMessage.setAttribute("newMessage", lastMessage.getAttribute("newMessage") + "<br>" + message);
                
                // Update the timestamp
                lastMessage.setAttribute("timestamp", timestamp.toString());
                
                // Scroll it into view
                this.scrollToNewItem(lastMessage);
                return;
            }
        }
    }
    
    // Create a new message block
    var item = this.scene.createBox("conversationHistoryItem");
    if (item)
    {
        // Add the properties to this message block
        item.setAttribute("buddyId", buddyId);
        item.setAttribute("time", this.getTimeString());
        item.setAttribute("newMessage", message);
        item.setAttribute("identity", identity);
        item.setAttribute("timestamp", timestamp.toString());
        if (isAutoResponse)
        {
            item.setAttribute("autoResponse", "true");
        }
        if (!lastMessage)
        {
            // The first message doesn't need the horizontal rule
            item.setAttribute("showRule", "false");
        }
        
        // Add the new message block and scroll to it
        this._gadget.addContentChild(item);
        this.scrollToNewItem(item);
    }
}

component.prototype.addOfflineMessage = function(buddyId, message, identity, msgRcvdTS)
{
    var timestamp = this.getTimestamp();
    var elapsedMins = Math.round((timestamp - msgRcvdTS) / 60);
    if (elapsedMins < 0)
    {
        elapsedMins = 1;
    }
    
    // Create a new message block
    var item = this.scene.createBox("conversationHistoryItem");
    if (item)
    {
        // Add the properties to this message block
        item.setAttribute("buddyId", buddyId);
        
        message = "[Offline IM sent " + elapsedMins + " minutes ago] <br>" + message;
        item.setAttribute("newMessage", message);
        item.setAttribute("identity", identity);
        item.setAttribute("timestamp", timestamp.toString());
        
        // Add the new message block and scroll to it
        this._gadget.addContentChild(item);
        this.scrollToNewItem(item);
    }
}

component.prototype.scrollToNewItem = function(item)
{
    // We have to force a layout since we likely just added an item and it hasn't done a layout yet
    this.scene.layout();
    
    // TODO - replace this with this._gadget.scrollToEnd();
    if (this._gadget.overflowed)
    {
        this._gadget.scrollOffsetY = 99999;
    }
}

component.prototype.getTimestamp = function()
{
    var date = new Date();
    return date.getTime() / 1000;
}

component.prototype.getTimeString = function()
{
    var date = new Date();
    var hours = date.getHours();
    var mins = date.getMinutes();
    var timestamp = ((hours > 12) ? (hours-12) : hours) + ":" + (mins < 10 ? "0"+mins : mins);
    return timestamp;
}

component.prototype.getLastMessage = function()
{
    return this._gadget.lastContentChild;
}

component.prototype.getFocusedBox = function()
{
    var child = this._gadget.firstContentChild;
    while (child)
    {
        if (child.getAttribute("focused") == "true")
        {
            return child;
        }
        child = child.nextSibling;
    }
    return null;
}

component.prototype.parseMessage = function(message)
{
    newMessage = this.stripHTML(message);
    newMessage = parseSmileys(newMessage);
    return newMessage;
}

component.prototype.stripHTML = function(string)
{
    var newString = string.replace(/(<br\/>)/ig,"&lt;br&gt; ");
    newString = newString.replace(/(<([^>]+)>)/ig,"");
    
    newString = newString.replace("<", "&lt;");
    newString = newString.replace(">", "&gt;");
    
    return newString;
}

