﻿// ****************************************************************************
//
// Copyright © 2008 AOL LLC  All rights reserved.
// This software contains valuable confidential and proprietary information
// of AOL LLC and is subject to applicable licensing agreements.
// Unauthorized reproduction, transmission or distribution of this file and
// its contents is a violation of applicable laws.
//
//           A O L  L L C   C O N F I D E N T I A L
//
// ****************************************************************************
//
//  updateChecker
//
//  This object handles checking for updates and storing any update information.
//  This object also contains all knowledge of how often updates should be checked, when 
//  they were last checked, etc.
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.updateCheckInterval = 86400000; //ms in 24 hours
updateChecker.prototype.updateCheckTimer = null;
updateChecker.prototype.xmlhttp = null;

function updateChecker()
{
}

function updateChecker_rsc_global(context)
{
    if (context)
    {
        context.onReadyStateChange();
    }
}

//////////////////////////////////////////////////////////////////////////////

updateChecker.prototype.onReadyStateChange = function() 
{
    if (this.xmlhttp.readyState==4) 
    {
        if (this.xmlhttp.status == 200)
        {            
            scene.updateObj.clearUpdateInfo();   //clear any previous update info 
                         
            responseLength = this.xmlhttp.getResponseHeader("Content-Length");
            if (responseLength > 0)
            {
                var responseText = this.xmlhttp.responseText;
                var replyArray = responseText.split(";");

                //shell.app.debugLog("Software update info: ");
                //shell.app.debugLog("displayUrl: " + replyArray[0]);
                //shell.app.debugLog("downloadUrl: " + replyArray[1]);
                //shell.app.debugLog("description: " + replyArray[2]);
                //shell.app.debugLog("mandatoryFlag: " + replyArray[3]);
                
                displayUrl = makeSafeUrl(replyArray[0]);
                downloadUrl = makeSafeUrl(replyArray[1]);
                updateDescription = replyArray[2];
                updateMandatoryFlag = replyArray[3];
                
                if ((displayUrl.length > 0))
                {
                    //shell.app.debugLog("Software update is available...");
                    
                    //write the info into prefs object...

                    var kvs = new keyValueStorage();
                    var storeName = scene.prefObj.prefsGetGlobalStore();
                    kvs.open( storeName );
                    kvs.setValue(scene.prefObj.updateDisplayUrlKey,displayUrl);
                    kvs.setValue(scene.prefObj.updateDownloadUrlKey,downloadUrl);
                    kvs.setValue(scene.prefObj.updateDescriptionKey,updateDescription);
                    
                    if (updateMandatoryFlag.toLowerCase() == "true")
                        kvs.setValue(scene.prefObj.updateMandatoryFlagKey,"true");
                    else
                        kvs.setValue(scene.prefObj.updateMandatoryFlagKey,"false");
                        
                    kvs.close();
                    kvs = null;
                }
                else
                {
                    //shell.app.debugLog("got back empty string - no update...");
                }
            } 
            scene.updateObj.setNextUpdateCheck(); //update check was successful - set timestamp for next attempt                     
        }
        else
        {
            //shell.app.debugLog("Got back error on http request = status: " + this.xmlhttp.status);
        }
    }
}


//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.isCriticalUpdateAvailable = function()
{
    var hasCriticalUpdate = false;
    var mandatoryFlag = "";
    
    //see if we have any mandatory updates waiting...
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        mandatoryFlag = kvs.getValue(scene.prefObj.updateMandatoryFlagKey,"");
        kvs.close( storeName );
        kvs = null;
    }
    catch(e)
    { 
    }  
    
    if (mandatoryFlag == "true")
    {
        hasCriticalUpdate = true;
    }
    return hasCriticalUpdate;
}

//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.isUpdateAvailable = function()
{
    var updateAvailable = false;
    var displayUrl = "";
    
    //see if we have any updates waiting...
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        displayUrl = kvs.getValue(scene.prefObj.updateDisplayUrlKey,"");
        kvs.close( storeName );
        kvs = null;
    }
    catch(e)
    { 
    }   
    
    if (displayUrl != "")
    {
        updateAvailable = true;
    }
    return updateAvailable;
}

//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.getDisplayUrl = function()
{
    var displayUrl;
    
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        displayUrl = kvs.getValue(scene.prefObj.updateDisplayUrlKey,"");
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
    
    return displayUrl;
}

//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.getDownloadUrl = function()
{
    var downloadUrl;
    
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        downloadUrl = kvs.getValue(scene.prefObj.updateDownloadUrlKey,"");
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }

    return downloadUrl;
}

//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.getDescription = function()
{
    var description;
  
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        description = kvs.getValue(scene.prefObj.updateDescriptionKey,"");
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
    return description;
}

//////////////////////////////////////////////////////////////////////////////
//doUpdateCheck
//
// fired by timer...
//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.doUpdateCheck = function()
{
    //shell.app.debugLog("In doUpdateCheck!");
    
    var shouldCheck = this.shouldCheckForUpdate();

    if (shouldCheck)
    {
        //shell.app.debugLog("It's time to check for our next update!");
        this.checkForUpdates(shell.app.productName, shell.app.buildName, shell.app.version, shell.app.locale, shell.app.platform);
    }
    
    if (this.updateCheckTimer != null)
    {
        this.updateCheckTimer.cancel();
        this.updateCheckTimer = null;
    }
    
    this.updateCheckTimer = setTimeoutObject(this, "doUpdateCheck", this.updateCheckInterval + 1000); //make sure this fires at least a second after the time we stamped for "next update check"
}


///////////////////////////////////////////////////////////////////////////////
// checkForUpdates
//
// params:
// productName  - product name
// buildName    - CM product name
// version      - version number of product
// locale       - ex: "en-US"
// platform     - ex: "Windows NT 6.0"
//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.checkForUpdates = function(productName, buildName, version, locale, platform)
{  
    //shell.app.debugLog("Checking for updates");

    var updateServer = "http://mobileupd.web.aol.com/mobileUpdates/checkForUpdates.php";
    
    //Check to see if we should be using the test/staging server....
    var useTestServer = "";
    var useDevServer = "";
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        useTestServer = kvs.getValue(scene.prefObj.updateUseTestServerKey,"");
        useDevServer = kvs.getValue(scene.prefObj.updateUseDevServerKey,"");
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
    
    if (useTestServer != "") //use test server - reset updateServer variable
    {
        updateServer = "http://mobileupd-stg.web.aol.com/mobileUpdates/checkForUpdates.php";
    }
    
    if (useDevServer != "") //use dev unit test server - reset updateServer variable
    {
        updateServer = "http://mobileupd-dev.web.aol.com/mobileUpdates/checkForUpdates.php";
    }        
    
    var userAgent = shell.app.userAgent;    
    var updateRequestUrl = updateServer + "?" + "locale=" + encodeURIComponent(locale) + "&" + "product=" + encodeURIComponent(buildName) + "&" + "version=" + encodeURIComponent(version) + "&" + "platform=" + encodeURIComponent(platform) + "&" + "format=text";
     
    //shell.app.debugLog(updateRequestUrl);
     
    var displayUrl = "";
    var downloadUrl = "";
    var updateDescription = "";
    var updateMandatoryFlag = "";
    try 
    {
        this.xmlhttp = null;
        this.xmlhttp = new XMLHttpRequest();
        var self = this;
        this.xmlhttp.open("GET", updateRequestUrl);
        this.xmlhttp.setRequestHeader("If-Modified-Since", "Thu, 1 Jan 1970 00:00:00 GMT");  //always fetch it
        this.xmlhttp.setRequestHeader("Cache-Control", "no-cache");
        if (userAgent.length > 0)  
        {
            this.xmlhttp.setRequestHeader("User-Agent", userAgent);
        }
        this.xmlhttp.onreadystatechange=function () { updateChecker_rsc_global(self) };
        this.xmlhttp.send(null);
    }
    catch(e)
    { 

    }
}
///////////////////////////////////////////////////////////////////////////////
// clearUpdateInfo
//
// Clear any update information out of storage.  
//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.clearUpdateInfo = function()
{   
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        kvs.removeKey(scene.prefObj.updateDisplayUrlKey);
        kvs.removeKey(scene.prefObj.updateDownloadUrlKey);
        kvs.removeKey(scene.prefObj.updateDescriptionKey);
        kvs.removeKey(scene.prefObj.updateMandatoryFlagKey);
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
}


///////////////////////////////////////////////////////////////////////////////
// setNextUpdateCheck
//
// Persists the next time we can do an update.  
//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.setNextUpdateCheck = function()
{    
    var curDate = new Date();
    var nextCheckTime = new Date(curDate.getTime() + updateChecker.prototype.updateCheckInterval);
    var nextCheckUTCTime = nextCheckTime.toUTCString();
        
    try 
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        kvs.setValue(scene.prefObj.updateNextCheckKey,nextCheckUTCTime);
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
                                
}

///////////////////////////////////////////////////////////////////////////////
// clearNextUpdateCheck
// 
// Clears persisted time for next update check.
//
//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.clearUpdateCheck = function()
{    
    try
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        kvs.removeKey(scene.prefObj.updateNextCheckKey);
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
}

///////////////////////////////////////////////////////////////////////////////
// shouldCheckForUpdate
//
// If a preference was set using setNextUpdateCheck - checks current time against
// that time and returns true if the current time is later.  If no time has
// been persisted, this function also returns true.
//
//////////////////////////////////////////////////////////////////////////////
updateChecker.prototype.shouldCheckForUpdate = function()
{
    var shouldCheck = true;
    var curTime = new Date();   //current time
    
    var nextUpdateTimeStr = "";
    try 
    {
        var kvs = new keyValueStorage();
        var storeName = scene.prefObj.prefsGetGlobalStore();
        kvs.open( storeName );
        nextUpdateTimeStr = kvs.getValue(scene.prefObj.updateNextCheckKey,"");
        kvs.close();
        kvs = null;
    }
    catch(e)
    { 
    }
    
    //shell.app.debugLog("Got next check time: " + nextUpdateTimeStr);
    
    if (nextUpdateTimeStr != "")
    {
        var nextUpdateTime = new Date(nextUpdateTimeStr);
        if (nextUpdateTime.getTime() > curTime.getTime())
        {
            shouldCheck = false;
        }
    }
    
    //shell.app.debugLog("Returning from shouldCheckForUpdate: " + shouldCheck);
    return shouldCheck;
}

updateChecker.prototype.turnOffAlerts = function(buttonIndex)
{
    if (buttonIndex == 0)
    {
        try 
        {
            var kvs = new keyValueStorage();
            var storeName = scene.prefObj.prefsGetStoreForCurrentIdentity();
            kvs.open( storeName );
            kvs.setValue(scene.prefObj.updateNotifyKey, "false");
            kvs.close();
            kvs = null;
        } catch(e) {}
    }
    scene.getBoxById("activeConversationBar").removeBuddy("1_update");
}

updateChecker.prototype.launchLink = function(linkIndex)
{
    var link;
    switch(linkIndex)
    {
        case 0:
        {
            link = this.getDownloadUrl();
        }
        break;
        case 1:
        {
            link = this.getDisplayUrl();
        }
        break;
    }
    if (link)
    {
        try
        {
            appUtils.shellExecute(link);
        }
        catch(e){}
    }
}

//////////////////////////////////////////////////////////////////////////////

updateChecker.prototype.release = function()
{
    if (this.xmlhttp)
    {
        this.xmlhttp = null;
    }
}
